@@ -113,7 +113,7 @@ def forall(*lines):
     # Notation used by the monad implementation for the bind and sequence
     # operators, with any relevant whitespace.
     bind = " >> "
-    seq  = ".then"
+    seq = ".then"
 
     class env:
         def __init__(self):
@@ -142,14 +142,14 @@ def forall(*lines):
     begin_is_open = False
     for j, item in enumerate(lines):
         is_first = (j == 0)
-        is_last  = (j == len(lines) - 1)
+        is_last = (j == len(lines) - 1)
 
         if isinstance(item, Assignment):
             name, body = item
         else:
             name, body = None, item
         if name and not name.isidentifier():
-            raise ValueError("name must be valid identifier, got '{}'".format(name))
+            raise ValueError("name must be valid identifier, got {}".format(repr(name)))
         bodys.append(body)
 
         freevars = names.copy()  # names from the surrounding scopes
@@ -163,8 +163,8 @@ def forall(*lines):
         if callable(body):
             try:
                 if not arity_includes(body, 1):
-                    raise ValueError("Arity mismatch; callable body must allow arity 1, to take in the environment.")
-            except UnknownArity:
+                    raise TypeError("Arity mismatch; callable body must allow arity 1, to take in the environment.")
+            except UnknownArity:  # pragma: no cover
                 pass
             code = "monadify(bodys[{j:d}](e), {flag:s})".format(flag=unpack_flag, j=j)
         else:  # doesn't need the environment
@@ -214,9 +214,14 @@ def monadify(value, unpack=True):
             pass  # fall through
     return MonadicList(value)  # unit(List, value)
 
-class MonadicList:
-    """The List monad."""
-    def __init__(self, *elts):  # unit: x: a -> M a
+class MonadicList:  # TODO: This if anything is **the** place to use @typed.
+    """A monadic list."""
+    def __init__(self, *elts):
+        """The unit operator. Lift value(s) into a MonadicList.
+
+        *elts: a or [a]
+        returns: M a
+        """
         # Accept the sentinel nil as a special **item** that, when passed to
         # the List constructor, produces an empty list.
         if len(elts) == 1 and elts[0] is nil:
@@ -224,57 +229,130 @@ class MonadicList:
         else:
             self.x = elts
 
-    def __rshift__(self, f):  # bind: x: (M a), f: (a -> M b)  -> (M b)
+    def __rshift__(self, f):
+        """Monadic bind; standard notation ">>=" in Haskell.
+
+        self: M a
+        f: a -> M b
+        returns: M b
+
+        Generally speaking, bind is defined as::
+            m >> f = m.fmap(f).join()
+
+        Specifically for `MonadicList`, bind is `flatmap`.
+        """
         # bind ma f = join (fmap f ma)
         return self.fmap(f).join()
         # done manually, essentially List.from_iterable(flatmap(lambda elt: f(elt), self.x))
         #return List.from_iterable(result for elt in self.x for result in f(elt))
 
-    # Sequence a.k.a. "then"; standard notation ">>" in Haskell.
-    def then(self, f):  # self: M a,  f : M b  -> M b
+    def then(self, f):
+        """Sequence, a.k.a. "then"; standard notation ">>" in Haskell.
+
+        Like `bind`, but discarding the input `a`.
+
+        self: M a
+        f : M b
+        returns: M b
+        """
         cls = self.__class__
         if not isinstance(f, cls):
             raise TypeError("Expected a List monad, got {} with data {}".format(type(f), f))
         return self >> (lambda _: f)
 
     @classmethod
-    def guard(cls, b):  # bool -> List   (for the list monad)
+    def guard(cls, b):
+        """Allow a branch of the computation to continue only if `b` is truthy.
+
+        b: bool
+        returns: M b
+
+        How to use:
+          - The type of `guard` is (bool -> M b). You'll want to wrap it in a function
+            that takes in an `a`; then `guard` outputs the `M b`, as expected by monadic
+            bind, so that you can bind your MonadicList `m` to your guard function.
+          - The call to `guard` produces a dummy `MonadicList`, which will be non-blank
+            (with exactly one item) if `b` is truthy, and blank if `b` is falsey.
+          - Use `.then(...)` just after the `guard` to discard the dummy, and replace with
+            the actual output you want. The value (that passed the guard) from the original
+            `MonadicList` is still live in the current scope.
+              - If you just want to filter, just `MonadicList(x)` it (recall that here the
+                constructor stands for the `unit` operator).
+          - When an input doesn't pass the guard, the blank output from `guard` automatically
+            cancels the rest of that branch of the computation.
+        """
         if b:
             return cls(True)  # List with one element; value not intended to be actually used.
         return cls()  # 0-element List; short-circuit this branch of the computation.
 
-    def __getitem__(self, i): # make List iterable so that "for result in f(elt)" works
-        return self.x[i]      # (when f outputs a List monad)
+    # make List iterable so that "for result in f(elt)" works (when f outputs a List monad)
+    def __iter__(self):
+        return iter(self.x)
+    def __len__(self):
+        return len(self.x)
+    def __getitem__(self, i):
+        return self.x[i]
 
-    def __add__(self, other): # concatenation of Lists, for convenience
+    def __eq__(self, other):
+        if other is self:
+            return True
+        if len(self) != len(other):
+            return False
+        return other == self.x
+
+    def __add__(self, other):
+        """Concatenation of MonadicList, for convenience."""
+        if not isinstance(other, MonadicList):
+            raise TypeError("Expected a monadic list, got {} with value {}".format(type(other), repr(other)))
         cls = self.__class__
         return cls.from_iterable(self.x + other.x)
 
-    def __str__(self):
+    def __repr__(self):  # pragma: no cover
         clsname = self.__class__.__name__
-        return "<{} {}>".format(clsname, self.x)
+        return "{}{}".format(clsname, self.x)
 
     @classmethod
-    def from_iterable(cls, iterable):  # convenience
+    def from_iterable(cls, iterable):
+        """Convenience method: turn an iterable into a MonadicList.
+
+        Eager; the input iterable will be iterated over in its entirety
+        to produce the list. If it is consumable, it will be consumed.
+        """
         try:
             return cls(*iterable)
-        except TypeError: # maybe a generator; try forcing it before giving up.
+        except TypeError:  # maybe a generator; try forcing it before giving up.
             return cls(*tuple(iterable))
 
     def copy(self):
+        """Return a copy of this MonadicList."""
         cls = self.__class__
         return cls(*self.x)
 
-    # Lift a regular function into a List-producing one.
     @classmethod
-    def lift(cls, f):         # lift: f: (a -> b)  -> (a -> M b)
+    def lift(cls, f):
+        """Lift a regular function into a List-producing one.
+
+        f: a -> b
+        returns: a -> M b
+        """
         return lambda x: cls(f(x))
 
-    def fmap(self, f):        # fmap: x: (M a), f: (a -> b)  -> (M b)
+    def fmap(self, f):
+        """The map operator.
+
+        self: M a
+        f: a -> b
+        returns: M b
+        """
         cls = self.__class__
         return cls.from_iterable(f(elt) for elt in self.x)
 
-    def join(self):           # join: x: M (M a)  -> M a
+    def join(self):
+        """The join operator. Flatten nested self.
+
+        x: M (M a)
+        returns: M a
+        """
         cls = self.__class__
         if not all(isinstance(elt, cls) for elt in self.x):
             raise TypeError("Expected a nested List monad, got {}".format(self.x))
@@ -282,10 +360,11 @@ class MonadicList:
         return cls.from_iterable(elt for sublist in self.x for elt in sublist)
 
 insist = MonadicList.guard  # retroactively require expr to be True
-def deny(expr):      # end a branch of the computation if expr is True
-    return insist(not expr)
+def deny(v):
+    """Opposite of `insist`. End a branch of the computation if `v` is truthy."""
+    return insist(not v)
 
 # TODO: export these or not? insist and deny already cover the interesting usage.
 # anything with one item (except nil), actual value is not used
-ok = ("ok",)         # let the computation proceed (usually alternative to fail)
-fail = ()            # end a branch of the computation
+ok = ("ok",)  # let the computation proceed (usually alternative to fail)
+fail = ()     # end a branch of the computation
