@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 """Additional containers and container utilities."""
 
-__all__ = ["box", "frozendict", "roview", "view", "ShadowedSequence",
+__all__ = ["box", "ThreadLocalBox", "unbox", "Some", "Shim",
+           "frozendict", "roview", "view", "ShadowedSequence",
+           "mogrify",
            "get_abcs", "in_slice", "index_in_slice",
            "SequenceView", "MutableSequenceView"]  # ABCs
 
@@ -9,16 +11,18 @@ from functools import wraps
 from itertools import repeat
 from abc import abstractmethod
 from collections import abc
-from collections.abc import Container, Iterable, Hashable, Sized, \
-                            Sequence, Mapping, Set, \
-                            MutableSequence, MutableMapping, MutableSet
+from collections.abc import (Container, Iterable, Hashable, Sized,
+                             Sequence, Mapping, Set,
+                             MutableSequence, MutableMapping, MutableSet,
+                             MappingView)
 from inspect import isclass
 from operator import lt, le, ge, gt
+import threading
 
-from .llist import cons
+from .llist import cons, Nil
 from .misc import getattrrec
 from .env import env
-from .dynassign import _Env
+from .dynassign import _Dyn
 
 def get_abcs(cls):
     """Return a set of the collections.abc superclasses of cls (virtuals too)."""
@@ -27,7 +31,6 @@ def get_abcs(cls):
 # TODO: allow multiple input container args in mogrify, like map does (also support longest, fillvalue)
 #   OTOH, that's assuming an ordered iterable... so maybe not for general containers?
 # TODO: move to unpythonic.it? This is a spork...
-_dictitems_type = type({}.items())
 def mogrify(func, container):
     """In-place recursive map for mutable containers.
 
@@ -62,8 +65,7 @@ def mogrify(func, container):
 
         - The ``cons`` container from ``unpythonic.llist`` (including the
           ``llist`` linked lists). This is treated with the general tree
-          strategy, so nested linked lists will be flattened, and the
-          final ``nil`` is also processed.
+          strategy. Any ``nil`` is passed through as-is.
 
     Any value that does not match any of these is treated as an atom.
 
@@ -78,17 +80,21 @@ def mogrify(func, container):
             y = [doit(elt) for elt in x]
             if hasattr(x, "clear"):
                 x.clear()  # list has this, but not guaranteed by MutableSequence
-            else:
+            else:  # pragma: no cover, no realistic `MutableSequence` that is missing `clear`?
                 while x:
                     x.pop()
             x.extend(y)
             return x
+        elif isinstance(x, MutableSequenceView):  # our own cat food
+            y = [doit(elt) for elt in x]
+            x[:] = y
+            return x
         elif isinstance(x, MutableSet):
             y = {doit(elt) for elt in x}
             x.clear()
             if hasattr(x, "update"):
                 x.update(y)  # set has this, but not guaranteed by MutableSet
-            else:
+            else:  # pragma: no cover, no realistic `MutableSet` that is missing `update`?
                 for elt in y:
                     x.add(elt)
             return x
@@ -99,15 +105,28 @@ def mogrify(func, container):
             x.clear()
             x.update(y)
             return x
-        elif isinstance(x, box):
-            x.x = doit(x.x)  # unfortunate attr name :)
+        elif isinstance(x, (box, ThreadLocalBox)):
+            x.set(doit(x.get()))
             return x
         # immutable containers
+        elif isinstance(x, Nil):  # for unpythonic.llist.ll() support
+            return x
         elif isinstance(x, cons):
             return cons(doit(x.car), doit(x.cdr))
+        elif isinstance(x, Some):
+            return Some(doit(x.get()))
+        elif isinstance(x, SequenceView):  # our own cat food
+            ctor = type(getattrrec(x, "seq"))  # de-onionize
+            return ctor(doit(elt) for elt in x)
+        # dict_items and similar cannot be instantiated, and they support only iteration,
+        # not in-place modification, so return a regular set
+        # (this turns up in "with curry" blocks using somedict.items() as a function argument,
+        #  due to the maybe_force_args() in curry)
+        elif isinstance(x, MappingView):
+            return {doit(elt) for elt in x}
         # env and dyn provide the Mapping API, but shouldn't get the general Mapping treatment here.
         # (This is important for the curry and lazify macros.)
-        elif isinstance(x, Mapping) and not isinstance(x, (env, _Env)):
+        elif isinstance(x, Mapping) and not isinstance(x, (env, _Dyn)):
             ctor = type(x)
             return ctor({k: doit(v) for k, v in x.items()})
         elif isinstance(x, Sequence) and not isinstance(x, (str, bytes, range)):
@@ -116,10 +135,7 @@ def mogrify(func, container):
             ctor = cls._make if hasattr(cls, "_make") else cls
             return ctor(doit(elt) for elt in x)
         elif isinstance(x, Set):
-            # dict_items cannot be instantiated, so return a regular set if we're asked to mogrify one
-            # TODO: is it possible to get the owner dict and mogrify it instead?
-            # (this case usually turns up in "with curry" with somedict.items())
-            ctor = type(x) if not isinstance(x, _dictitems_type) else set
+            ctor = type(x)
             return ctor({doit(elt) for elt in x})
         return func(x)  # atom
     return doit(container)
@@ -141,12 +157,12 @@ class box:
 
         b = box(17)
         def f(b):
-            b.x = 23  # yes!
+            b << 23  # yes!
         f(b)
-        print(b.x)  # 23
+        print(unbox(b))  # 23
 
-    (It's called ``x`` and not ``value`` to minimize the number of additional
-    keystrokes needed.)
+    This is the recommended unpythonic syntax. If you like OOP, you can
+    `b.set(23)` instead of `b << 23`, and `b.get()` instead of `unbox(b)`.
 
     In terms of ``collections.abc``, a ``box`` is a ``Container``, ``Iterable``
     and ``Sized``. A box is **not** hashable, because it is a mutable container.
@@ -169,7 +185,7 @@ class box:
     """
     def __init__(self, x=None):
         self.x = x
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "box({})".format(repr(self.x))
     def __contains__(self, x):
         return self.x == x
@@ -179,6 +195,180 @@ class box:
         return 1
     def __eq__(self, other):
         return other == self.x
+    def set(self, x):
+        """Store a new value in the box, replacing the old one.
+
+        As a convenience, returns the new value.
+
+        Since a function call is an expression, you can use this form
+        also in a lambda, or indeed in any expression position.
+        """
+        self.x = x
+        return x
+    def __lshift__(self, x):
+        """Syntactic sugar for storing a new value.
+
+        `b << 42` is the same as `b.set(42)`.
+
+        (Note that for `env`, the `<<` operator returns the *environment* so it
+        can be chained to make several assignments, but that doesn't make sense
+        for a `box`, so we just return the new value.)
+        """
+        return self.set(x)
+    def get(self):
+        """Return the value currently in the box.
+
+        The syntactic sugar for `b.get()` is `unbox(b)`.
+        """
+        return self.x
+
+# We re-implement instead of making `box` use an `env` as a place
+# so that the thread-locality feature is pay-as-you-go (no loss in
+# performance for the regular, non-thread-local `box`.)
+class ThreadLocalBox(box):
+    """Like box, but the store is thread-local.
+
+    The initially provided `x` is used as the default object, which serves as
+    the initial contents of the box in all threads. (Note what this implies if
+    that `x` happens to be mutable.)
+    """
+    def __init__(self, x=None):
+        self.storage = threading.local()
+        self._default = x
+    def __repr__(self):  # pragma: no cover
+        """**WARNING**: the repr shows only the content seen by the current thread."""
+        return "ThreadLocalBox({})".format(repr(self.get()))
+    def __contains__(self, x):
+        return self.get() == x
+    def __iter__(self):
+        return (x for x in (self.get(),))
+    def __eq__(self, other):
+        return other == self.get()
+    def set(self, x):
+        self.storage.x = x
+        return x
+    def __lshift__(self, x):
+        return self.set(x)
+    def get(self):
+        if hasattr(self.storage, "x"):  # default overridden in this thread?
+            return self.storage.x
+        return self._default
+    def setdefault(self, x):
+        """Change the default object."""
+        self._default = x
+    def getdefault(self):
+        """Get the default object."""
+        return self._default
+    def clear(self):
+        """Remove the value in the box in this thread, thus unshadowing the default."""
+        if hasattr(self.storage, "x"):
+            del self.storage.x
+
+class Some:
+    """Explicitly represent thing-ness as opposed to nothingness.
+
+    Useful for optional fields. Using a `Some` container makes it possible to
+    tell apart the presence of a `None` value from the absence of a value::
+
+        x = Some(42)    # we have a value, it's `42`
+        x = Some(None)  # we have a value, it's `None`
+        x = None        # we don't have a value
+
+    In a way, `Some` is a relative of `box`: it's an **immutable** single-item
+    container. It supports `.get` and `unbox`, but no `<<` or `.set`.
+    """
+    def __init__(self, x=None):
+        self.x = x
+    def __repr__(self):  # pragma: no cover
+        return "Some({})".format(repr(self.x))
+    def __contains__(self, x):
+        return self.x == x
+    def __iter__(self):
+        return (x for x in (self.x,))
+    def __len__(self):
+        return 1
+    def __eq__(self, other):
+        return other == self.x
+    def get(self):
+        """Return the value currently in the `Some`.
+
+        The syntactic sugar for `b.get()` is `unbox(b)`.
+        """
+        return self.x
+
+def unbox(b):
+    """Return the value from inside the box b.
+
+    Syntactic sugar for `b.get()`.
+
+    If `b` is not a `box` (or `ThreadLocalBox` or `Some`), raises `TypeError`.
+    """
+    if not isinstance(b, (box, Some)):
+        raise TypeError("Expected box, got {} with value {}".format(type(b), repr(b)))
+    return b.get()
+
+class Shim:
+    """Attribute access proxy.
+
+    Hold a target object inside a box. When an attribute of the shim
+    is accessed (whether to get or set it), redirect that access to
+    the object that is currently inside the box.
+
+    The target may be replaced at any time, simply by sending a new value
+    into the box instance you gave to the `Shim` constructor.
+
+    Another use case is to combo with `ThreadLocalBox`, e.g. to redirect
+    stdin/stdout only when used from some specific threads.
+
+    Since deep down, attribute access is the whole point of objects, `Shim` is
+    essentially a transparent object proxy. (For example, a method call is an
+    attribute read (via a descriptor), followed by a function call.)
+
+    thebox:   a `box` instance that holds the target object. The box must
+              be created manually, to maximize composability: a `box` or
+              `ThreadLocalBox` can be chosen as appropriate for the
+              particular use case.
+
+    fallback: optional; a fallback object. Either any object (not boxed),
+              or a `box` instance (in case you want to replace the fallback
+              later).
+
+              **For attribute reads** (i.e. `__getattr__`), if the object in
+              the box does not have the requested attribute, `Shim` will try
+              to get it from the fallback. If `fallback` is boxed, the
+              attribute read takes place on the object in the box. If it is
+              not boxed, the attribute read takes place directly on `fallback`.
+
+              Any **attribute writes** (i.e. `__setattr__`, binding or rebinding
+              an attribute) always take place on the primary target object
+              in `thebox`.
+
+    If you need to chain fallbacks, this can be done with `foldr`::
+
+        boxes = [box(obj) for obj in ...]
+        *others, final_fallback = boxes
+        s = foldr(Shim, final_fallback, others)
+
+    Here `Shim(box, fallback)` is foldr's `op(elt, acc)`.
+    """
+    def __init__(self, thebox, fallback=None):
+        if not isinstance(thebox, box):
+            raise TypeError("Expected box, got {} with value {}".format(type(thebox), repr(thebox)))
+        self._shim_box = thebox
+        self._shim_fallback = fallback
+    def __getattr__(self, k):
+        thing = unbox(self._shim_box)
+        fallback = self._shim_fallback
+        if not fallback or hasattr(thing, k):
+            return getattr(thing, k)
+        # fallback and not hasattr(thing, k)
+        otherthing = unbox(fallback) if isinstance(fallback, box) else fallback
+        return getattr(otherthing, k)
+    def __setattr__(self, k, v):
+        if k in ("_shim_box", "_shim_fallback"):
+            return super().__setattr__(k, v)
+        thing = unbox(self._shim_box)
+        return setattr(thing, k, v)
 
 _the_empty_frozendict = None
 class frozendict:
@@ -209,7 +399,7 @@ class frozendict:
 
     Just like for ``tuple`` and ``frozenset``, the empty ``frozendict`` is a
     singleton; each no-argument call ``frozendict()`` will return the same object
-    instance. (But don't pickle it; it is freshly created in each session).
+    instance.
 
     In terms of ``collections.abc``, a ``frozendict`` is a ``Container``,
     ``Hashable``, ``Iterable``, ``Mapping`` and ``Sized``.
@@ -218,7 +408,9 @@ class frozendict:
     detected by the built-ins ``issubclass`` and ``isinstance``, but they are
     not part of the MRO.
     """
-    def __new__(cls, *ms, **bindings):  # make the empty frozendict() a singleton
+    # Make the empty frozendict() a singleton, but allow invoking the constructor
+    # multiple times, always returning the same instance.
+    def __new__(cls, *ms, **bindings):
         if not ms and not bindings:
             global _the_empty_frozendict
             if _the_empty_frozendict is None:
@@ -226,6 +418,19 @@ class frozendict:
             return _the_empty_frozendict
         return super().__new__(cls)  # object() takes no args, but we need to see them
 
+    # Pickling support.
+    # https://github.com/Technologicat/unpythonic/issues/55
+    # https://docs.python.org/3/library/pickle.html#object.__getnewargs_ex__
+    # https://docs.python.org/3/library/pickle.html#object.__getnewargs__
+    def __getnewargs__(self):
+        if self is not _the_empty_frozendict:
+            # In our case it doesn't matter what the value is, as long as there is one,
+            # because `__new__` uses the *presence* of any args to know the instance is
+            # nonempty, and hence an instance should actually be created instead of
+            # just returning the empty singleton frozendict.
+            return ("nonempty",)
+        return ()
+
     def __init__(self, *ms, **bindings):
         """Arguments:
 
@@ -246,14 +451,11 @@ class frozendict:
         super().__init__()
         self._data = {}
         for m in ms:
-            try:
-                self._data.update(m)
-            except TypeError:
-                pass
+            self._data.update(m)
         self._data.update(bindings)
 
     @wraps(dict.__repr__)
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "frozendict({})".format(self._data.__repr__())
 
     def __hash__(self):
@@ -304,6 +506,8 @@ for abscls in get_abcs(dict) - {MutableMapping} | {Hashable}:
     abscls.register(frozendict)
 for abscls in (Container, Iterable, Sized):
     abscls.register(box)
+    abscls.register(ThreadLocalBox)
+    abscls.register(Some)
 del abscls  # namespace cleanup
 
 # -----------------------------------------------------------------------------
@@ -328,21 +532,21 @@ class MutableSequenceView(SequenceView):
     """
     @abstractmethod
     def __setitem__(self, k, v):
-        pass
+        pass  # pragma: no cover
     @abstractmethod
     def reverse(self):
-        pass
+        pass  # pragma: no cover
 
 # -----------------------------------------------------------------------------
 
 class _StrReprEqMixin:
-    def _lowlevel_repr(self):
+    def _lowlevel_repr(self):  # pragma: no cover
         cls = type(getattrrec(self, "seq"))  # de-onionize
         ctor = tuple if hasattr(cls, "_make") else cls  # slice of namedtuple -> tuple
         return ctor(x for x in self)
-    def __str__(self):
+    def __str__(self):  # pragma: no cover
         return str(self._lowlevel_repr())
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "{:s}({!r})".format(self.__class__.__name__, self._lowlevel_repr())
 
     def __eq__(self, other):
@@ -350,8 +554,8 @@ class _StrReprEqMixin:
             return True
         if len(self) != len(other):
             return False
-        for v, w in zip(self, other):
-            if v != w:
+        for v1, v2 in zip(self, other):
+            if v1 != v2:
                 return False
         return True
 
@@ -439,11 +643,11 @@ class roview(SequenceView, _StrReprEqMixin):
                 return ctor(self.seq, k)
             return ctor(self, k)
         elif isinstance(k, tuple):
-            raise TypeError("multidimensional subscripting not supported; got '{}'".format(k))
+            raise TypeError("multidimensional subscripting not supported; got {}".format(repr(k)))
         else:
             data, r = self._update_cache()
-            l = len(r)
-            if k >= l or k < -l:
+            n = len(r)
+            if k >= n or k < -n:
                 raise IndexError("view index out of range")
             return data[r[k]]
 
@@ -505,10 +709,10 @@ class view(roview, MutableSequenceView):
             for j, item in zip(r[k], vs):
                 data[j] = item
         elif isinstance(k, tuple):
-            raise TypeError("multidimensional subscripting not supported; got '{}'".format(k))
+            raise TypeError("multidimensional subscripting not supported; got {}".format(repr(k)))
         else:
-            l = len(r)
-            if k >= l or k < -l:
+            n = len(r)
+            if k >= n or k < -n:
                 raise IndexError("view assigment index out of range")
             data[r[k]] = v
     def reverse(self):
@@ -516,8 +720,7 @@ class view(roview, MutableSequenceView):
 
 # -----------------------------------------------------------------------------
 
-# TODO: is there a way to inherit from an ABC without causing the metaclass to change to ABCMeta?
-# TODO: (we want the default implementations of e.g. count, index in the MRO; but this is a concrete class.)
+# Inherit from Sequence, because we want the default implementations of e.g. count, index to be found in the MRO.
 class ShadowedSequence(Sequence, _StrReprEqMixin):
     """Sequence with some elements shadowed by those from another sequence.
 
@@ -545,10 +748,10 @@ class ShadowedSequence(Sequence, _StrReprEqMixin):
     def __iter__(self):
         if self.ix is None:  # allow no-op ShadowedSequences since the repr suggests one could do that
             return iter(self.seq)
-        l = len(self)
+        n = len(self)
         getone = self._getone
         def ShadowedSequenceIterator():
-            for j in range(l):
+            for j in range(n):
                 yield getone(j)
         return ShadowedSequenceIterator()
 
@@ -558,31 +761,32 @@ class ShadowedSequence(Sequence, _StrReprEqMixin):
     def __getitem__(self, k):
         if self.ix is None:  # allow no-op ShadowedSequences since the repr suggests one could do that
             return self.seq[k]
-        l = len(self)
+        n = len(self)
         if isinstance(k, slice):
             cls = type(self.seq)
             ctor = tuple if hasattr(cls, "_make") else cls  # slice of namedtuple -> tuple
-            return ctor(self._getone(j) for j in range(l)[k])
+            return ctor(self._getone(j) for j in range(n)[k])
         elif isinstance(k, tuple):
-            raise TypeError("multidimensional subscripting not supported; got '{}'".format(k))
+            raise TypeError("multidimensional subscripting not supported; got {}".format(repr(k)))
         else:
-            if k >= l or k < -l:
+            if k >= n or k < -n:
                 raise IndexError("ShadowedSequence index out of range")
             return self._getone(k)
 
     def _getone(self, k):
         ix = self.ix
-        l = len(self)
-        if in_slice(k, ix, l):
+        n = len(self)
+        if in_slice(k, ix, n):
             if isinstance(ix, int):
                 return self.v  # just one item
             # we already know k is in ix, so skip validation for speed.
-            i = _index_in_slice(k, ix, l, _validate=False)
+            i = _index_in_slice(k, ix, n, _validate=False)
             if i >= len(self.v):
                 raise IndexError("Replacement sequence too short; attempted to access index {} with len {} (items: {})".format(i, len(self.v), self.v))
             return self.v[i]
         return self.seq[k]  # not in slice
 
+# TODO: fix flake8 E741 ambiguous variable name "l". Here it's part of the public API, so we'll have to wait until 15.0 to change the name.
 def in_slice(i, s, l=None):
     """Return whether the int i is in the slice s.
 
@@ -612,6 +816,7 @@ def in_slice(i, s, l=None):
     on_grid = (i - start) % step == 0
     return at_or_after_start and on_grid and before_stop
 
+# TODO: fix flake8 E741 ambiguous variable name "l". Here it's part of the public API, so we'll have to wait until 15.0 to change the name.
 def index_in_slice(i, s, l=None):
     """Return the index of the int i in the slice s, or None if i is not in s.
 
@@ -623,55 +828,69 @@ def index_in_slice(i, s, l=None):
 
 # efficiency: allow skipping the validation check for call sites
 # that have already checked with in_slice().
-def _index_in_slice(i, s, l=None, _validate=True):
-    if (not _validate) or in_slice(i, s, l):
-        wrap = _make_negidx_converter(l)
-        start, _, step = _canonize_slice(s, l, wrap)
+def _index_in_slice(i, s, n=None, _validate=True):  # n: length of sequence being indexed
+    if (not _validate) or in_slice(i, s, n):
+        wrap = _make_negidx_converter(n)
+        start, _, step = _canonize_slice(s, n, wrap)
         return (wrap(i) - start) // step
 
-def _make_negidx_converter(l):  # l: length of sequence being indexed
-    if l is not None:
-        if not isinstance(l, int):
-            raise TypeError("l must be int, got {} with value {}".format(type(l), l))
-        if l <= 0:
-            raise ValueError("l must be an int >= 1, got {}".format(l))
+def _make_negidx_converter(n):  # n: length of sequence being indexed
+    if n is not None:
+        if not isinstance(n, int):
+            raise TypeError("n must be int, got {} with value {}".format(type(n), n))
+        if n <= 0:
+            raise ValueError("n must be an int >= 1, got {}".format(n))
         def apply_conversion(k):
-            return k % l
+            return k % n
     else:
         def apply_conversion(k):
-            raise ValueError("Need l to interpret negative indices")
+            raise ValueError("Need n to interpret negative indices")
     def convert(k):
         if k is not None:
             if not isinstance(k, int):
-                raise TypeError("k must be int, got {} with value {}".format(type(k), k))
+                # This is not triggered in the current code because the outer
+                # layers protect against having to check here, but since the
+                # `convert` function is returned to the caller, let's be
+                # careful.
+                raise TypeError("k must be int, got {} with value {}".format(type(k), k))  # pragma: no cover
+            # Almost standard semantics for negative indices. Usually -n < k < n,
+            # but here we must allow for conversion of the end position, for
+            # which the last valid value is one past the end.
+            if n is not None and not -n <= k <= n:
+                raise IndexError("Should have -n <= k <= n, but n = {}, and k = {}".format(n, k))
             return apply_conversion(k) if k < 0 else k
     return convert
 
-def _canonize_slice(s, l=None, w=None):  # convert negatives, inject defaults.
+def _canonize_slice(s, n=None, wrap=None):  # convert negatives, inject defaults.
     if not isinstance(s, slice):
-        raise TypeError("s must be slice, got {} with value {}".format(type(s), s))
+        # Not triggered in the current code, because this is an internal function
+        # and `in_slice` already checks; but let's be careful in case this is later
+        # used elsewhere. (And, it's already possible that some internal caller
+        # incorrectly uses the no-check mode of the internal implementation function
+        # `_index_in_slice`.)
+        raise TypeError("s must be slice, got {} with value {}".format(type(s), s))  # pragma: no cover
 
     step = s.step if s.step is not None else +1  # no "s.step or +1"; someone may try step=0
     if step == 0:
         raise ValueError("slice step cannot be zero")  # message copied from range(5)[0:4:0]
 
-    wrap = w or _make_negidx_converter(l)
+    wrap = wrap or _make_negidx_converter(n)
 
     start = wrap(s.start)
     if start is None:
         if step > 0:
             start = 0
         else:
-            if l is None:
-                raise ValueError("Need l to determine default start for step < 0")
+            if n is None:
+                raise ValueError("Need n to determine default start for step < 0")
             start = wrap(-1)
 
     stop = wrap(s.stop)
     if stop is None:
         if step > 0:
-            if l is None:
-                raise ValueError("Need l to determine default stop for step > 0")
-            stop = l
+            if n is None:
+                raise ValueError("Need n to determine default stop for step > 0")
+            stop = n
         else:
             stop = -1  # yes, really -1 to have index 0 inside the slice
 
