@@ -9,21 +9,27 @@ Memoize is typical FP (Racket has it in mischief), and flip comes from Haskell.
 
 __all__ = ["memoize", "curry", "iscurried",
            "flip", "rotate",
-           "apply", "identity", "const", "notf", "andf", "orf",
+           "apply", "identity", "const",
+           "notf", "andf", "orf",
            "composer1", "composel1", "composer1i", "composel1i",  # single arg
-           "composer",  "composel",  "composeri",  "composeli",   # multi-arg
+           "composer", "composel", "composeri", "composeli",   # multi-arg
            "composerc", "composelc", "composerci", "composelci",  # multi-arg w/ curry
            "to1st", "to2nd", "tokth", "tolast", "to",
            "withself"]
 
 from functools import wraps, partial
-from operator import itemgetter
 
-from .arity import arities, UnknownArity
+from .arity import arities, resolve_bindings, tuplify_bindings, UnknownArity
 from .fold import reducel
 from .dynassign import dyn, make_dynvar
 from .regutil import register_decorator
+from .symbol import sym
 
+# we use @passthrough_lazy_args (and handle possible lazy args) to support unpythonic.syntax.lazify.
+from .lazyutil import passthrough_lazy_args, islazy, force, force1, maybe_force_args
+
+_success = sym("_success")
+_fail = sym("_fail")
 @register_decorator(priority=10)
 def memoize(f):
     """Decorator: memoize the function f.
@@ -37,35 +43,36 @@ def memoize(f):
     **CAUTION**: ``f`` must be pure (no side effects, no internal state
     preserved between invocations) for this to make any sense.
     """
-    success, fail = [object() for _ in range(2)]
     memo = {}
     @wraps(f)
     def memoized(*args, **kwargs):
-        k = (args, tuple(sorted(kwargs.items(), key=itemgetter(0))))
+        k = tuplify_bindings(resolve_bindings(f, *args, **kwargs))
         if k not in memo:
             try:
-                result = (success, f(*args, **kwargs))
+                result = (_success, maybe_force_args(f, *args, **kwargs))
             except BaseException as err:
-                result = (fail, err)
+                result = (_fail, err)
             memo[k] = result  # should yell separately if k is not a valid key
         kind, value = memo[k]
-        if kind is fail:
+        if kind is _fail:
             raise value
         return value
+    if islazy(f):
+        memoized = passthrough_lazy_args(memoized)
     return memoized
 
 #def memoize_simple(f):  # essential idea, without exception handling
 #    memo = {}
 #    @wraps(f)
 #    def memoized(*args, **kwargs):
-#        k = (args, tuple(sorted(kwargs.items(), key=itemgetter(0))))
+#        k = tuplify_bindings(resolve_bindings(f, *args, **kwargs))
 #        if k not in memo:
 #            memo[k] = f(*args, **kwargs)
 #        return memo[k]
 #    return memoized
 
 make_dynvar(curry_context=[])
-make_dynvar(_curry_allow_uninspectable=False)  # if True, no-op if not inspectable.
+@passthrough_lazy_args
 def _currycall(f, *args, **kwargs):
     """Co-operate with unpythonic.syntax.curry.
 
@@ -73,11 +80,15 @@ def _currycall(f, *args, **kwargs):
     to ``curry(f)``, but definitions can be curried as usual.
 
     Hence we provide this separate mode to curry-and-call even if no args.
-    """
-    return curry(f, *args, _curry_force_call=True, **kwargs)
 
-@register_decorator(priority=90)
-def curry(f, *args, _curry_force_call=False, **kwargs):
+    This mode also no-ops when ``f`` is not inspectable, instead of raising
+    an ``unpythonic.arity.UnknownArity`` exception.
+    """
+    return curry(f, *args, _curry_force_call=True, _curry_allow_uninspectable=True, **kwargs)
+
+@register_decorator(priority=8)
+@passthrough_lazy_args
+def curry(f, *args, _curry_force_call=False, _curry_allow_uninspectable=False, **kwargs):
     """Decorator: curry the function f.
 
     Essentially, the resulting function automatically chains partial application
@@ -163,35 +174,54 @@ def curry(f, *args, _curry_force_call=False, **kwargs):
         from unpythonic import curry, composel, drop, take
 
         with_n = lambda *args: (partial(f, n) for n, f in args)
-        look = lambda n1, n2: composel(*with_n((n1, drop), (n2, take)))
-        assert tuple(curry(look, 5, 10, range(20))) == tuple(range(5, 15))
+        clip = lambda n1, n2: composel(*with_n((n1, drop), (n2, take)))
+        assert tuple(curry(clip, 5, 10, range(20))) == tuple(range(5, 15))
+
+    **CAUTION**: BUG: `curry` may fail to actually call the function even after
+    sufficient arguments have been collected, if some of the positional-or-keyword
+    arguments of the function being curried are passed by name (in the first call).
+    It seems those arguments don't reduce the expected remaining positional arity,
+    although they should. See issue #61:
+        https://github.com/Technologicat/unpythonic/issues/61
+
+    **Workaround**: if possible, at the definition site for your function, declare
+    any arguments you plan to pass by name as keyword-only; then they won't affect
+    the positional arity.
     """
+    f = force(f)  # lazify support: we need the value of f
+    # trivial case first: interaction with call_ec and other replace-def-with-value decorators
+    if not callable(f):
+        return f
     # trivial case first: prevent stacking curried wrappers
     if iscurried(f):
         if args or kwargs or _curry_force_call:
-            return f(*args, **kwargs)
+            return maybe_force_args(f, *args, **kwargs)
         return f
     # TODO: improve: all required name-only args should be present before calling f.
     # Difficult, partial() doesn't remove an already-set kwarg from the signature.
     try:
         min_arity, max_arity = arities(f)
     except UnknownArity:  # likely a builtin
-        if not dyn._curry_allow_uninspectable:  # usual behavior
+        if not _curry_allow_uninspectable:  # usual behavior
             raise
         # co-operate with unpythonic.syntax.curry; don't crash on builtins
         if args or kwargs or _curry_force_call:
-            return f(*args, **kwargs)
+            return maybe_force_args(f, *args, **kwargs)
         return f
     @wraps(f)
     def curried(*args, **kwargs):
         outerctx = dyn.curry_context
         with dyn.let(curry_context=(outerctx + [f])):
             if len(args) < min_arity:
-                return curry(partial(f, *args, **kwargs))
+                p = partial(f, *args, **kwargs)
+                if islazy(f):
+                    p = passthrough_lazy_args(p)
+                return curry(p)
             # passthrough on right, like https://github.com/Technologicat/spicy
             if len(args) > max_arity:
                 now_args, later_args = args[:max_arity], args[max_arity:]
-                now_result = f(*now_args, **kwargs)  # use up all kwargs now
+                now_result = maybe_force_args(f, *now_args, **kwargs)  # use up all kwargs now
+                now_result = force(now_result) if not isinstance(now_result, tuple) else force1(now_result)
                 if callable(now_result):
                     # curry it now, to sustain the chain in case we have
                     # too many (or too few) args for it.
@@ -206,11 +236,13 @@ def curry(f, *args, _curry_force_call=False, **kwargs):
                 if isinstance(now_result, tuple):
                     return now_result + later_args
                 return (now_result,) + later_args
-            return f(*args, **kwargs)
+            return maybe_force_args(f, *args, **kwargs)
+    if islazy(f):
+        curried = passthrough_lazy_args(curried)
     curried._is_curried_function = True  # stash for detection
     # curry itself is curried: if we get args, they're the first step
     if args or kwargs or _curry_force_call:
-        return curried(*args, **kwargs)
+        return maybe_force_args(curried, *args, **kwargs)
     return curried
 
 def iscurried(f):
@@ -230,7 +262,9 @@ def flip(f):
     """Decorator: flip (reverse) the positional arguments of f."""
     @wraps(f)
     def flipped(*args, **kwargs):
-        return f(*reversed(args), **kwargs)
+        return maybe_force_args(f, *reversed(args), **kwargs)
+    if islazy(f):
+        flipped = passthrough_lazy_args(flipped)
     return flipped
 
 def rotate(k):
@@ -254,12 +288,17 @@ def rotate(k):
             n = len(args)
             if not n:
                 raise TypeError("Expected at least one argument")
+            if not -n < k < n:  # standard semantics for negative indices
+                raise IndexError("Should have -n < k < n, but n = len(args) = {}, and k = {}".format(n, k))
             j = -k % n
             rargs = args[-j:] + args[:-j]
-            return f(*rargs, **kwargs)
+            return maybe_force_args(f, *rargs, **kwargs)
+        if islazy(f):
+            rotated = passthrough_lazy_args(rotated)
         return rotated
     return rotate_k
 
+@passthrough_lazy_args
 def apply(f, arg0, *more, **kwargs):
     """Scheme/Racket-like apply.
 
@@ -275,13 +314,16 @@ def apply(f, arg0, *more, **kwargs):
 
     The ``**kwargs`` are passed to `f`, allowing to pass also named arguments.
     """
+    f = force(f)
     if not more:
         args, lst = (), tuple(arg0)
     else:
         args = (arg0,) + more[:-1]
         lst = tuple(more[-1])
-    return f(*(args + lst), **kwargs)
+    return maybe_force_args(f, *(args + lst), **kwargs)
 
+# Not marking this as lazy-aware works better with continuations (since this
+# is the default cont, and return values should be values, not lazy[])
 def identity(*args):
     """Identity function.
 
@@ -293,9 +335,14 @@ def identity(*args):
 
         assert identity(1, 2, 3) == (1, 2, 3)
         assert identity(42) == 42
+        assert identity() is None
     """
+    if not args:
+        return None
     return args if len(args) > 1 else args[0]
 
+# In lazify, return values are always just values, so we have to force args
+# to compute the return value; as a shortcut, just don't mark this as lazy.
 def const(*args):
     """Constant function.
 
@@ -307,8 +354,18 @@ def const(*args):
         c = const(1, 2, 3)
         assert c(42, "foo") == (1, 2, 3)
         assert c("anything") == (1, 2, 3)
+        assert c() == (1, 2, 3)
+
+        c = const(42)
+        assert c("anything") == 42
+
+        c = const()
+        assert c("anything") is None
     """
-    ret = args if len(args) > 1 else args[0]
+    if not args:
+        ret = None
+    else:
+        ret = args if len(args) > 1 else args[0]
     def constant(*a, **kw):
         return ret
     return constant
@@ -322,7 +379,9 @@ def notf(f):  # Racket: negate
         assert notf(lambda x: 2*x)(0) is True
     """
     def negated(*args, **kwargs):
-        return not f(*args, **kwargs)
+        return not maybe_force_args(f, *args, **kwargs)
+    if islazy(f):
+        negated = passthrough_lazy_args(negated)
     return negated
 
 def andf(*fs):  # Racket: conjoin
@@ -343,10 +402,12 @@ def andf(*fs):  # Racket: conjoin
     def conjoined(*args, **kwargs):
         b = True
         for f in fs:
-            b = b and f(*args, **kwargs)
+            b = b and maybe_force_args(f, *args, **kwargs)
             if not b:
                 return False
         return b
+    if all(islazy(f) for f in fs):
+        conjoined = passthrough_lazy_args(conjoined)
     return conjoined
 
 def orf(*fs):  # Racket: disjoin
@@ -369,15 +430,18 @@ def orf(*fs):  # Racket: disjoin
     def disjoined(*args, **kwargs):
         b = False
         for f in fs:
-            b = b or f(*args, **kwargs)
+            b = b or maybe_force_args(f, *args, **kwargs)
             if b:
                 return b
         return False
+    if all(islazy(f) for f in fs):
+        disjoined = passthrough_lazy_args(disjoined)
     return disjoined
 
 def _make_compose1(direction):  # "left", "right"
     def compose1_two(f, g):
-        return lambda x: f(g(x))
+        # return lambda x: f(g(x))
+        return lambda x: maybe_force_args(f, maybe_force_args(g, x))
     if direction == "right":
         compose1_two = flip(compose1_two)
     def compose1(fs):
@@ -392,7 +456,10 @@ def _make_compose1(direction):  # "left", "right"
         # Using reducel is particularly nice here:
         #  - if fs is empty, we output None
         #  - if fs contains only one item, we output it as-is
-        return reducel(compose1_two, fs)  # op(elt, acc)
+        composed = reducel(compose1_two, fs)  # op(elt, acc)
+        if all(islazy(f) for f in fs):
+            composed = passthrough_lazy_args(composed)
+        return composed
     return compose1
 
 _compose1_left = _make_compose1("left")
@@ -440,18 +507,21 @@ def _make_compose(direction):  # "left", "right"
                 # to the function that is applied second.
                 bindings = {"curry_context": dyn.curry_context + [composed]}
             with dyn.let(**bindings):
-                a = g(*args)
+                a = maybe_force_args(g, *args)
             # we could duck-test, but this is more predictable for the user
             # (consider chaining functions that manipulate a generator), and
             # tuple specifically is the pythonic multiple-return-values thing.
             if isinstance(a, tuple):
-                return f(*a)
-            return f(a)
+                return maybe_force_args(f, *a)
+            return maybe_force_args(f, a)
         return composed
     if direction == "right":
         compose_two = flip(compose_two)
     def compose(fs):
-        return reducel(compose_two, fs)  # op(elt, acc)
+        composed = reducel(compose_two, fs)  # op(elt, acc)
+        if all(islazy(f) for f in fs):
+            composed = passthrough_lazy_args(composed)
+        return composed
     return compose
 
 _compose_left = _make_compose("left")
@@ -518,21 +588,24 @@ def tokth(k, f):
 
     Negative indices also supported.
 
-    Especially useful in multi-arg compose chains. See ``test()`` for examples.
+    Especially useful in multi-arg compose chains.
+    See ``unpythonic.test.test_fun`` for examples.
     """
     def apply_f_to_kth_arg(*args):
         n = len(args)
         if not n:
             raise TypeError("Expected at least one argument")
-        j = k % n  # handle also negative values
-        m = j + 1
-        if n < m:
-            raise TypeError("Expected at least {:d} arguments, got {:d}".format(m, n))
+        if not -n < k < n:  # standard semantics for negative indices
+            raise IndexError("Should have -n < k < n, but n = len(args) = {}, and k = {}".format(n, k))
+        j = k % n  # --> j ∈ {0, 1, ..., n - 1}, even if k < 0
+        m = j + 1  # --> m ∈ {1, 2, ..., n}
         out = list(args[:j])
-        out.append(f(args[j]))  # mth argument
+        out.append(maybe_force_args(f, args[j]))  # mth argument
         if n > m:
             out.extend(args[m:])
         return tuple(out)
+    if islazy(f):
+        apply_f_to_kth_arg = passthrough_lazy_args(apply_f_to_kth_arg)
     return apply_f_to_kth_arg
 
 def to1st(f):
@@ -601,5 +674,8 @@ def withself(f):
     """
     @wraps(f)
     def fwithself(*args, **kwargs):
-        return f(fwithself, *args, **kwargs)
+        #return f(fwithself, *args, **kwargs)
+        return maybe_force_args(f, fwithself, *args, **kwargs)  # support unpythonic.syntax.lazify
+    if islazy(f):
+        fwithself = passthrough_lazy_args(fwithself)
     return fwithself
