@@ -10,7 +10,7 @@ For more batteries for itertools, see also the ``unpythonic.fold`` module.
 ``itertools`` recipes.
 """
 
-__all__ = ["rev", "map_longest",
+__all__ = ["rev", "map", "map_longest",
            "rmap", "rzip", "rmap_longest", "rzip_longest",
            "mapr", "zipr", "mapr_longest", "zipr_longest",
            "flatmap",
@@ -18,13 +18,20 @@ __all__ = ["rev", "map_longest",
            "take", "drop", "split_at",
            "unpack",
            "tail", "butlast", "butlastn",
-           "first", "second", "nth", "last",
-           "scons",
+           "first", "second", "nth", "last", "lastn",
+           "scons", "pad",
            "flatten", "flatten1", "flatten_in",
            "iterate", "iterate1",
            "partition",
-           "inn", "iindex"]
+           "partition_int",
+           "inn", "iindex", "find",
+           "window", "chunked",
+           "within", "fixpoint",
+           "interleave",
+           "subset", "powerset",
+           "allsame"]
 
+from builtins import map as stdlib_map
 from operator import itemgetter
 from itertools import tee, islice, zip_longest, starmap, chain, filterfalse, groupby, takewhile
 from collections import deque
@@ -49,6 +56,22 @@ def rev(iterable):
     except TypeError:
         return reversed(tuple(iterable))
 
+def map(function, iterable0, *iterables):
+    """Curry-friendly map.
+
+    Thin wrapper around Python's builtin ``map``, making it mandatory to
+    provide at least one iterable, so we may say things such as::
+
+        from unpythonic import map, curry
+        oneplus = lambda x: 1 + x  # noqa: E731
+
+        add_one = curry(map, oneplus)
+
+        assert tuple(add_one(range(5))) == tuple(range(1, 6))
+
+    """
+    return stdlib_map(function, iterable0, *iterables)
+
 # When completing an existing set of functions (map, zip, zip_longest),
 # consistency wins over curry-friendliness.
 def map_longest(func, *iterables, fillvalue=None):
@@ -181,18 +204,18 @@ def flatmap(f, iterable0, *iterables):
             else:
                 s = x**0.5
                 return (s, -s)
-        assert tuple(flatmap(msqrt, (0, 1, 4, 9))) == \\
-               (0., 1., -1., 2., -2., 3., -3.)
+        assert (tuple(flatmap(msqrt, (0, 1, 4, 9))) ==
+                (0., 1., -1., 2., -2., 3., -3.))
 
         def add_and_tuplify(a, b):
             return (a + b,)
-        assert tuple(flatmap(add_and_tuplify, (10, 20, 30), (1, 2, 3))) == \\
-               (11, 22, 33)
+        assert (tuple(flatmap(add_and_tuplify, (10, 20, 30), (1, 2, 3))) ==
+                (11, 22, 33))
 
         def sum_and_diff(a, b):
             return (a + b, a - b)
-        assert tuple(flatmap(sum_and_diff, (10, 20, 30), (1, 2, 3))) == \\
-               (11, 9, 22, 18, 33, 27)
+        assert (tuple(flatmap(sum_and_diff, (10, 20, 30), (1, 2, 3))) ==
+                (11, 9, 22, 18, 33, 27))
     """
     yield from chain.from_iterable(map(f, iterable0, *iterables))
 #    for xs in map(f, iterable0, *iterables):
@@ -253,15 +276,16 @@ def drop(n, iterable):
 
     This is ``consume`` from ``itertools`` recipes.
     """
+    if n is None:
+        it = iter(iterable)
+        deque(it, maxlen=0)
+        return it
     if not isinstance(n, int):
         raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     it = iter(iterable)
-    if n is None:
-        deque(it, maxlen=0)
-    else:
-        next(islice(it, n, n), None)  # advance it to empty slice starting at n
+    next(islice(it, n, n), None)  # advance it to empty slice starting at n
     return it
 
 def split_at(n, iterable):
@@ -373,14 +397,19 @@ def butlastn(n, iterable):
     Uses intermediate storage - do not use the original iterator after calling
     ``butlastn``.
     """
-    # we let StopIteration propagate from anything that could raise it here.
     it = iter(iterable)
     q = deque()
-    for _ in range(n+1):
-        q.append(next(it))
+    for _ in range(n + 1):
+        try:
+            q.append(next(it))
+        except StopIteration:
+            return
     while True:
         yield q.popleft()
-        q.append(next(it))
+        try:
+            q.append(next(it))
+        except StopIteration:
+            return
 
 def first(iterable, *, default=None):
     """Like nth, but return the first item."""
@@ -418,6 +447,20 @@ def last(iterable, *, default=None):
     d = deque(iterable, maxlen=1)  # C speed
     return d.pop() if d else default
 
+def lastn(n, iterable):
+    """Yield the last n items from an iterable.
+
+    We consume the iterable until it runs out of items, then return a generator
+    that yields up to ``n`` last items seen, in the original order.
+
+    If there are fewer than ``n`` items in the iterable, the generator yields
+    them all.
+
+    **Caution**: Will not terminate for infinite inputs.
+    """
+    d = deque(iterable, maxlen=n)  # C speed
+    yield from d
+
 def scons(x, iterable):
     """Prepend one element to the start of an iterable, return new iterable.
 
@@ -429,6 +472,21 @@ def scons(x, iterable):
     """
     return chain((x,), iterable)
 
+def pad(n, fillvalue, iterable):
+    """Pad iterable with copies of fillvalue so its length is at least ``n``.
+
+    Examples::
+
+        assert tuple(pad(5, None, range(3))) == (0, 1, 2, None, None)
+        assert tuple(pad(5, None, ())) == (None, None, None, None, None)
+        assert tuple(pad(5, None, range(6))) == tuple(range(6))
+    """
+    k = 0  # used if iterable is empty
+    for k, x in enumerate(iterable, start=1):
+        yield x
+    for _ in range(k, n):
+        yield fillvalue
+
 def flatten(iterable, pred=None):
     """Recursively remove nested structure from iterable.
 
@@ -475,10 +533,10 @@ def flatten_in(iterable, pred=None):
 
         is_nested = lambda e: all(isinstance(x, (list, tuple)) for x in e)
         data = (((1, 2), ((3, 4), (5, 6)), 7), ((8, 9), (10, 11)))
-        assert tuple(flatten(data, is_nested))    == \\
-               (((1, 2), ((3, 4), (5, 6)), 7), (8, 9), (10, 11))
-        assert tuple(flatten_in(data, is_nested)) == \\
-               (((1, 2), (3, 4), (5, 6), 7), (8, 9), (10, 11))
+        assert (tuple(flatten(data, is_nested)) ==
+                (((1, 2), ((3, 4), (5, 6)), 7), (8, 9), (10, 11)))
+        assert (tuple(flatten_in(data, is_nested)) ==
+                (((1, 2), (3, 4), (5, 6), 7), (8, 9), (10, 11)))
     """
     pred = pred or (lambda x: True)
     it = iter(iterable)
@@ -529,11 +587,72 @@ def partition(pred, iterable):
     (Example: partition the natural numbers, and only ever read the even numbers.
     It will eventually run out of memory storing all the odd numbers "to be read
     later".)
+
+    Not to be confused with `unpythonic.it.partition_int`, which partitions
+    a (small) positive integer to smaller integers, in all possible ways,
+    such that those integers sum to the original one.
     """
     # iterable is walked only once; tee handles the intermediate storage.
     t1, t2 = tee(iterable)
     return filterfalse(pred, t1), filter(pred, t2)
 
+def partition_int(n, lower=1, upper=None):
+    """Yield all ordered sequences of smaller positive integers that sum to `n`.
+
+    `n` must be an integer >= 1.
+
+    `lower` is an optional lower limit for each member of the sum. Each member
+    of the sum must be `>= lower`.
+
+    (Most of the splits are a ravioli consisting mostly of ones, so it is much
+    faster to not generate such splits than to filter them out from the result.
+    The default value `lower=1` generates everything.)
+
+    `upper` is, similarly, an optional upper limit; each member of the sum
+    must be `<= upper`. The default `None` means no upper limit (effectively,
+    in that case `upper=n`).
+
+    It must hold that `1 <= lower <= upper <= n`.
+
+    Not to be confused with `unpythonic.it.partition`, which partitions an
+    iterable based on a predicate.
+
+    **CAUTION**: The number of possible partitions grows very quickly with `n`,
+    so in practice this is only useful for small numbers, or with a lower limit
+    that is not too much smaller than `n / 2`. A possible use case for this
+    function is to determine the number of letters to allocate for each
+    component of an anagram that may consist of several words.
+
+    See:
+        https://en.wikipedia.org/wiki/Partition_(number_theory)
+    """
+    # sanity check the preconditions, fail-fast
+    if not isinstance(n, int):
+        raise TypeError('n must be integer; got {} with value {}'.format(type(n), repr(n)))
+    if not isinstance(lower, int):
+        raise TypeError('lower must be integer; got {} with value {}'.format(type(lower), repr(lower)))
+    if upper is not None and not isinstance(upper, int):
+        raise TypeError('upper must be integer; got {} with value {}'.format(type(upper), repr(lower)))
+    upper = upper if upper is not None else n
+    if n < 1:
+        raise ValueError('n must be positive; got {}'.format(n))
+    if lower < 1 or upper < 1 or lower > n or upper > n or lower > upper:
+        raise ValueError('it must hold that 1 <= lower <= upper <= n; got lower={}, upper={}'.format(lower, upper))
+
+    def _partition(n):
+        for k in range(min(n, upper), lower - 1, -1):
+            m = n - k
+            if m == 0:
+                yield (k,)
+            else:
+                out = []
+                for item in _partition(m):
+                    out.append((k,) + item)
+                for term in out:
+                    yield term
+
+    return _partition(n)  # instantiate the generator
+
 def inn(x, iterable):
     """Contains-check (``x in iterable``) with automatic termination.
 
@@ -586,14 +705,16 @@ def inn(x, iterable):
         y0 = next(it)
     except StopIteration:
         return False
-    if y0 == x: return True
+    if y0 == x:
+        return True
     yj = y0
     while yj == y0:
         try:
             yj = next(it)
         except StopIteration:
             return False
-    if yj == x: return True
+    if yj == x:
+        return True
     d = yj - y0
     assert d != 0
     pred = (lambda elt: elt <= x) if d > 0 else (lambda elt: elt >= x)
@@ -613,3 +734,284 @@ def iindex(x, iterable):
         if elt == x:
             return j
     raise ValueError("{} is not in iterable".format(x))
+
+def find(predicate, iterable, default=None):
+    """Return the first item matching `predicate` in `iterable`, or `default` if no match.
+
+    If you need all matching items, just use the builtin `filter` or a comprehension;
+    this is a convenience utility to get the first match only.
+    """
+    return next(filter(predicate, iterable), default)
+
+# TODO: in 0.15.0, maybe switch the argument order of window() for curry-friendliness?
+def window(iterable, n=2):
+    """Sliding length-n window iterator for a general iterable.
+
+    Acts like ``zip(s, s[1:], ..., s[n-1:])`` for a sequence ``s``, but the input
+    can be any iterable.
+
+    If there are fewer than ``n`` items in the input iterable, an empty iterator
+    is returned.
+
+    Inspired by ``with_next`` discussed in:
+
+        https://opensource.com/article/18/3/loop-better-deeper-look-iteration-python
+    """
+    if n < 2:
+        raise ValueError("expected n >= 2, got {}".format(n))
+    it = iter(iterable)
+    xs = deque()
+    for _ in range(n):
+        try:
+            xs.append(next(it))
+        except StopIteration:
+            def empty_iterable():
+                yield from ()
+            return empty_iterable()
+    def windowed():
+        while True:
+            yield tuple(xs)
+            xs.popleft()
+            try:
+                xs.append(next(it))
+            except StopIteration:
+                return
+    return windowed()
+
+def chunked(n, iterable):
+    """Split an iterable into constant-length chunks.
+
+    Conceptually, whereas ``window`` slides its stencil through which the
+    original iterable is viewed, ``chunked`` partitions the iterable with
+    no overlap between consecutive stencil positions.
+
+    This returns a generator that yields the chunks. Unlike ``window``, to
+    remain storage-agnostic, each chunk itself is represented as an iterator
+    (so if you want tuples, convert each chunk yourself - see example below).
+
+    No temporary storage is allocated, this is essentially a stream filter
+    built on itertools.
+
+    Example::
+        chunks = chunked(3, range(9))
+        assert [tuple(chunk) for chunk in chunks] == [(0, 1, 2), (3, 4, 5), (6, 7, 8)]
+        chunks = chunked(3, range(7))
+        assert [tuple(chunk) for chunk in chunks] == [(0, 1, 2), (3, 4, 5), (6,)]
+
+    Based on StackOverflow answers by Sven Marnach and reclosedev:
+        https://stackoverflow.com/questions/8991506/iterate-an-iterator-by-chunks-of-n-in-python
+    """
+    if n < 2:
+        raise ValueError("expected n >= 2, got {}".format(n))
+    it = iter(iterable)
+    def chunker():
+        try:
+            while True:
+                cit = islice(it, n)
+                # we need the next() to see the StopIteration when the first empty slice occurs
+                yield scons(next(cit), cit)
+        except StopIteration:
+            return
+    return chunker()
+
+def within(tol, iterable):
+    """Yield items from iterable until successive items are close enough.
+
+    Items are yielded until `abs(a - b) <= tol` for successive items
+    `a` and `b`.
+
+    If `tol == 0`, one final duplicate value will be yielded. This makes the
+    last two yielded values always satisfy the condition, even when `tol == 0`.
+
+    **CAUTION**: Intended for converging mathematical sequences, preferably
+    Cauchy sequences. Use on arbitrary input will lead to nasty surprises
+    (infinite output, or terminating the output early if a part of it looks
+    like a converging sequence; think a local maximum of `cos(x)`).
+    """
+    for a, b in window(iterable, n=2):
+        yield a
+        if abs(a - b) <= tol:
+            yield b
+            return
+
+def fixpoint(f, x0, tol=0):
+    """Compute the (arithmetic) fixed point of f, starting from the initial guess x0.
+
+    (Not to be confused with the logical fixed point with respect to the
+    definedness ordering.)
+
+    The fixed point must be attractive for this to work. See the Banach
+    fixed point theorem.
+    https://en.wikipedia.org/wiki/Banach_fixed-point_theorem
+
+    If the fixed point is attractive, and the values are represented in
+    floating point (hence finite precision), the computation should
+    eventually converge down to the last bit (barring roundoff or
+    catastrophic cancellation in the final few steps). Hence the default tol
+    of zero.
+
+    CAUTION: an arbitrary function from ℝ to ℝ **does not** necessarily
+    have a fixed point. Limit cycles and chaotic behavior of `f` will cause
+    non-termination. Keep in mind the classic example:
+    https://en.wikipedia.org/wiki/Logistic_map
+
+    Examples::
+        from math import cos, sqrt
+        from unpythonic import fixpoint, ulp
+        c = fixpoint(cos, x0=1)
+
+        # Actually "Newton's" algorithm for the square root was already known to the
+        # ancient Babylonians, ca. 2000 BCE. (Carl Boyer: History of mathematics)
+        def sqrt_newton(n):
+            def sqrt_iter(x):  # has an attractive fixed point at sqrt(n)
+                return (x + n / x) / 2
+            return fixpoint(sqrt_iter, x0=n / 2)
+        assert abs(sqrt_newton(2) - sqrt(2)) <= ulp(1.414)
+    """
+    return last(within(tol, iterate1(f, x0)))
+
+def interleave(*iterables):
+    """Interleave items from several iterables. Generator.
+
+    Example::
+
+        interleave(a, b, c) -> (a0, b0, c0, a1, b1, c1, ...)
+
+    until the shortest input runs out.
+    """
+    class ShortestInputEnded(Exception):
+        pass
+    iters = [iter(it) for it in iterables]
+    def roundrobin():
+        for it in iters:
+            try:
+                x = next(it)
+                yield x
+            except StopIteration:
+                raise ShortestInputEnded()
+    try:
+        while True:
+            yield from roundrobin()
+    except ShortestInputEnded:
+        return
+
+def subset(part, whole):
+    """Test whether `part` is a subset of `whole`.
+
+    Both must be iterable. Note consumable iterables will be consumed
+    by the test!
+
+    This is a convenience function.
+
+    Examples::
+
+        assert subset([1, 2, 3], [1, 2, 3, 4, 5])
+        assert subset({"cat"}, {"cat", "lynx"})
+    """
+    return all(elt in whole for elt in part)
+
+def powerset(iterable):
+    """Yield the powerset of a general iterable.
+
+    The powerset is the set of all subsets of items taken from the iterable.
+    Each subset (also single-item subsets) is packed as a tuple, to support
+    duplicate items in the input, as well as to make the output hashable
+    (provided that each item is). (This makes the output eligible for e.g.
+    `uniqify`.)
+
+    Works for general iterables, also potentially infinite ones, as long as
+    only a finite prefix is ever requested. But be aware that all the subsets
+    yielded so far are stored internally in order to form new subsets.
+
+    Examples::
+
+        tuple(powerset(range(3)))
+        # --> ((0,), (1,), (0, 1), (2,), (0, 2), (1, 2), (0, 1, 2))
+
+        # all divisors of 36 = 2 * 2 * 3 * 3
+        tuple(sorted(prod(xs) for xs in uniqify(powerset([2, 2, 3, 3]))))
+        # --> (2, 3, 4, 6, 9, 12, 18, 36)
+
+    If you want to try that for other positive integers, SymPy can perform
+    the factorization::
+
+        import sympy as sy
+        [factor for factor, multiplicity in sy.factorint(36).items()
+                for _ in range(multiplicity)]
+        # --> [2, 2, 3, 3]
+
+    **NOTE**: The itertools recipe implementation is shorter than ours, and
+    likely has better performance, but it assumes finite input; **we do not**.
+    The `more-itertools` library packages that recipe, so the same limitation
+    applies there, too. See:
+        https://docs.python.org/3/library/itertools.html
+        https://pypi.org/project/more-itertools/
+
+    **CAUTION**:
+
+    The size of the powerset of an iterable of length `n` is `2**n - 1`::
+
+        [len(list(powerset(range(k)))) for k in range(10)]
+        # --> [0, 1, 3, 7, 15, 31, 63, 127, 255, 511]
+
+    (proof by induction) and furthermore, the total item count in the subsets
+    also grows quickly::
+
+        from collections import Counter
+        def length_distribution(k):  # count of subsets of each length
+            return Counter(sorted(len(x) for x in powerset(range(k))))
+        def total_num_items(ld):
+            return sum(length * count for length, count in ld.items())
+        sizes = [total_num_items(length_distribution(k)) for k in range(10)]
+        # --> [0, 1, 4, 12, 32, 80, 192, 448, 1024, 2304]
+
+        d = length_distribution(10)
+        # --> Counter({1: 10,
+        #              2: 45,
+        #              3: 120,
+        #              4: 210,
+        #              5: 252,
+        #              6: 210,
+        #              7: 120,
+        #              8: 45,
+        #              9: 10,
+        #              10: 1})
+        total_num_items(d)
+        # --> 5120
+
+    Hence, building all of the power set becomes intractable quite quickly as
+    the length of the input iterable increases.
+    """
+    it = iter(iterable)
+    bag = []
+    while True:
+        try:
+            x = (next(it),)
+        except StopIteration:
+            return
+        yield x
+        t = [c + x for c in bag]
+        bag.append(x)
+        yield from t
+        bag.extend(t)
+
+def allsame(iterable):
+    """Return whether all elements of an iterable are the same.
+
+    The test uses `!=` to compare.
+
+    If `iterable` is empty, the return value is `True` (like for `all`).
+
+    If `iterable` has just one element, the return value is `True`.
+
+    **CAUTION**: Consumes consumable iterables.
+    """
+    it = iter(iterable)
+    try:
+        x0 = next(it)
+    except StopIteration:
+        return True  # like all(()) is True
+    for x in it:
+        if x != x0:
+            return False
+    return True
