@@ -18,7 +18,9 @@ Finally, we provide ready-made generators that yield some common sequences
 (currently, the Fibonacci numbers and the prime numbers).
 """
 
-__all__ = ["s", "m", "mg", "almosteq",
+__all__ = ["s", "imathify", "gmathify",
+           "m", "mg",  # old names, pre-0.14.3, will go away in 0.15.0
+           "almosteq",
            "sadd", "ssub", "sabs", "spos", "sneg", "sinvert", "smul", "spow",
            "struediv", "sfloordiv", "smod", "sdivmod",
            "sround", "strunc", "sfloor", "sceil",
@@ -26,49 +28,78 @@ __all__ = ["s", "m", "mg", "almosteq",
            "cauchyprod", "diagonal_reduce",
            "fibonacci", "primes"]
 
+from warnings import warn
 from itertools import repeat, takewhile, count
 from functools import wraps
-from operator import add as primitive_add, mul as primitive_mul, \
-                     pow as primitive_pow, mod as primitive_mod, \
-                     floordiv as primitive_floordiv, truediv as primitive_truediv, \
-                     sub as primitive_sub, \
-                     neg as primitive_neg, pos as primitive_pos, \
-                     and_ as primitive_and, xor as primitive_xor, or_ as primitive_or, \
-                     lshift as primitive_lshift, rshift as primitive_rshift, \
-                     invert as primitive_invert
+from operator import (add as primitive_add, mul as primitive_mul,
+                      pow as primitive_pow, mod as primitive_mod,
+                      floordiv as primitive_floordiv, truediv as primitive_truediv,
+                      sub as primitive_sub,
+                      neg as primitive_neg, pos as primitive_pos,
+                      and_ as primitive_and, xor as primitive_xor, or_ as primitive_or,
+                      lshift as primitive_lshift, rshift as primitive_rshift,
+                      invert as primitive_invert,
+                      lt as primitive_lt, le as primitive_le,
+                      eq as primitive_eq, ne as primitive_ne,
+                      ge as primitive_ge, gt as primitive_gt)
 
-from .it import take, rev
+from .it import take, rev, window
 from .gmemo import imemoize, gmemoize
 
+class _NoSuchType:
+    pass
+
 # stuff to support float, mpf and SymPy expressions transparently
 #
 from sys import float_info
 from math import log as math_log, copysign, trunc, floor, ceil
 try:
     from mpmath import mpf, almosteq as mpf_almosteq
-except ImportError:
-    mpf = mpf_almosteq = None
+except ImportError:  # pragma: no cover, optional at runtime, but installed at development time.
+    # Can't use a gensym here since `mpf` must be a unique *type*.
+    mpf = _NoSuchType
+    mpf_almosteq = None
 
 def _numsign(x):
+    """The sign function, for numeric inputs."""
     if x == 0:
         return 0
     return int(copysign(1.0, x))
 
 try:
     from sympy import log as _symlog, Expr as _symExpr, sign as _symsign
-    def log(x, b):
+    def log(x, b=None):
+        """The logarithm function.
+
+        Works for both numeric and symbolic (`SymPy.Expr`) inputs.
+
+        Default base `b=None` means `e`, i.e. take the natural logarithm.
+        """
         if isinstance(x, _symExpr):
             # https://stackoverflow.com/questions/46129259/how-to-simplify-logarithm-of-exponent-in-sympy
-            return _symlog(x, b).expand(force=True)
-        return math_log(x, b)
+            if b is not None:
+                return _symlog(x, b).expand(force=True)
+            else:
+                return _symlog(x).expand(force=True)
+        if b is not None:
+            return math_log(x, b)
+        else:
+            return math_log(x)
     def sign(x):
+        """The sign function.
+
+        Works for both numeric and symbolic (`SymPy.Expr`) inputs.
+        """
         if isinstance(x, _symExpr):
             return _symsign(x)
         return _numsign(x)
-except ImportError:
+except ImportError:  # pragma: no cover, optional at runtime, but installed at development time.
     log = math_log
     sign = _numsign
+    _symExpr = _NoSuchType
 
+# TODO: Overhaul `almosteq` in v0.15.0, should work like mpf for consistency.
+# TODO: Also move it to `unpythonic.misc`, where `ulp` already is. Or make a `numutil`.
 def almosteq(a, b, tol=1e-8):
     """Almost-equality that supports several formats.
 
@@ -92,12 +123,12 @@ def almosteq(a, b, tol=1e-8):
     if isinstance(a, mpf) and isinstance(b, mpf):
         return mpf_almosteq(a, b, tol)
     # compare as native float if only one is an mpf
-    elif isinstance(a, mpf) and isinstance(b, float):
+    elif isinstance(a, mpf) and isinstance(b, (float, int)):
         a = float(a)
-    elif isinstance(a, float) and isinstance(b, mpf):
+    elif isinstance(a, (float, int)) and isinstance(b, mpf):
         b = float(b)
 
-    if not all(isinstance(x, float) for x in (a, b)):
+    if not all(isinstance(x, (float, int)) for x in (a, b)):
         return False  # non-float type, already determined that a != b
     min_normal = float_info.min
     max_float = float_info.max
@@ -123,7 +154,8 @@ def s(*spec):
     *Convenience fallback*:
 
     As a fallback, we accept an explicit enumeration of all elements of the
-    desired sequence. This returns a genexpr that reads from a tuple. Syntax::
+    desired sequence. This returns a genexpr that reads from a tuple, but
+    adds infix math support. Syntax::
 
         s(1, 2, 3, 4, 5)
 
@@ -140,6 +172,23 @@ def s(*spec):
     syntax, because the number of terms cannot be computed from the value of the
     final element.
 
+    *Cyclic sequence*:
+
+    Convenience feature. Tag the repeating cycle of elements with a list
+    (must be a list, not a tuple). The list must have at least one element.
+    A final ``...``, after the list, is mandatory. Syntax::
+
+        s([*repeats], ...)
+        s(*initials, [*repeats], ...)
+
+    Examples::
+
+        s([1, 2], ...)        # --> 1, 2, 1, 2, 1, 2, ...
+        s(1, 2, [3, 4], ...)  # --> 1, 2, 3, 4, 3, 4, ... (3, 4 repeat)
+
+    Cyclic sequences **do not** support the optional-final-element termination
+    syntax.
+
     *Arithmetic sequence*: ``[a0, +d] -> a0, a0 + d, a0 + 2 d, ...``
 
     Two terms required, more allowed if consistent. Syntax::
@@ -216,10 +265,22 @@ def s(*spec):
 
     A symbolic example::
 
-        x = symbols("x", real=True)  # SymPy
-        px = lambda stream: stream * s(1, x, x**2, ...)  # powers of x
-        s1 = px(s(1, 3, 5, ...))  # 1, 3*x, 5*x**2, ...
-        s2 = px(s(2, 4, 6, ...))  # 2, 4*x, 6*x**2, ...
+        from sympy import symbols
+        x = symbols("x", real=True)
+        powers_of_x = lambda: s(1, x, x**2, ...)
+        univariate_polynomial = lambda coeffs: coeffs * powers_of_x()
+        s1 = univariate_polynomial(s(1, 3, 5, ...))  # 1, 3*x, 5*x**2, ...
+        s2 = univariate_polynomial(s(2, 4, 6, ...))  # 2, 4*x, 6*x**2, ...
+
+    In the example, the lambda is needed because the iterable produced
+    by `s(...)` is consumable; hence we must instantiate a new copy of
+    the powers of x each time `univariate_polynomial` is called.
+    We could also use `gmathify(imemoize(...))`:
+
+        powers_of_x = gmathify(imemoize(s(1, x, x**2, ...)))
+        univariate_polynomial = lambda coeffs: coeffs * powers_of_x()
+
+    The rest as above.
 
     **Notes**
 
@@ -250,45 +311,56 @@ def s(*spec):
             return False
 
     def analyze(*spec):  # raw spec (part before '...' if any) --> description
-        l = len(spec)
-        if l == 1:
+        n = len(spec)
+        if n == 1:
             a0 = spec[0]
             return ("const", a0, None)
-        elif l == 2:
+        elif n == 2:
             a0, a1 = spec
             d1 = a1 - a0
             if d1 == 0:
                 return ("const", a0, None)
             return ("arith", a0, d1)
-        elif l == 3:
+        elif n == 3:
             a0, a1, a2 = spec
             d1 = a1 - a0
             d2 = a2 - a1
             if d2 == d1 == 0:         # a0, a0, a0, ...             [a0, identity]
                 return ("const", a0, None)
             if almosteq(d2, d1):      # a0, a0 + d, a0 + 2 d, ...   [a0, +d]
-                d = (d1 + d2)/2  # average to give roundoff errors a chance to cancel
+                if d1 == d2:
+                    d = d1
+                else:
+                    # Note: even an arithmetic sequence will now become float.
+                    d = (d1 + d2) / 2  # average to give roundoff errors a chance to cancel
                 return ("arith", a0, d)
             if a0 != 0 and a1 != 0 and a2 != 0:
-                r1 = a1/a0
-                r2 = a2/a1
+                r1 = a1 / a0
+                r2 = a2 / a1
                 if almosteq(r2, r1):  # a0, a0*r, a0*r**2, ...      [a0, *r]
-                    r = (r1 + r2)/2
+                    if all(isinstance(a, int) for a in (a0, a1, a2)) and a1 // a0 == r1 and a2 // a1 == r2:
+                        r = a1 // a0
+                    else:
+                        # becomes float
+                        r = (r1 + r2) / 2
                     return ("geom", a0, r)
             if abs(a0) != 1 and a1 != 0 and a2 != 0:
                 p1 = log(abs(a1), abs(a0))
                 p2 = log(abs(a2), abs(a1))
                 if almosteq(p1, p2):  # a0, a0**p, (a0**p)**p, ...  [a0, **p]
-                    p = (p1 + p2)/2
+                    p = (p1 + p2) / 2
                     return ("power", a0, p)
+            # Most unrecognized sequences trigger this case.
             raise SyntaxError("Specification did not match any supported formula: '{}'".format(origspec))
         else:  # more elements are optional but must be consistent
-            data = [analyze(*triplet) for triplet in zip(spec, spec[1:], spec[2:])]
+            data = [analyze(*triplet) for triplet in window(iterable=spec, n=3)]
             seqtypes, x0s, ks = zip(*data)
-            def isconst(*xs):
+            def isconst(xs):
                 first, *rest = xs
                 return all(almosteq(x, first) for x in rest)
             if not isconst(seqtypes) or not isconst(ks):
+                # This case is only triggered if all triplets specify some
+                # recognized sequence, but the specifications don't agree.
                 raise SyntaxError("Inconsistent specification '{}'".format(origspec))
             return data[0]
 
@@ -303,35 +375,57 @@ def s(*spec):
             # elt = x0 + a*k --> a = (elt - x0) / k
             a = (elt - x0) / k
             if is_almost_int(a) and a > 0:
-                return 1 + round(a)  # fencepost
+                return int(1 + round(a))  # fencepost
         elif seqtype == "geom":
             # elt = x0*(k**a) --> k**a = (elt/x0) --> a = logk(elt/x0)
-            a = log(abs(elt/x0), abs(k))
+            a = log(abs(elt / x0), abs(k))
             if is_almost_int(a) and a > 0:
-                if not almosteq(x0*(k**a), elt):  # check parity of final term, could be an alternating sequence
+                if not almosteq(x0 * (k**a), elt):  # check parity of final term, could be an alternating sequence
                     return False
-                return 1 + round(a)
-        else: # seqtype == "power":
+                return int(1 + round(a))
+        else:  # seqtype == "power":
             # elt = x0**(k**a) --> k**a = logx0 elt --> a = logk (logx0 elt)
             a = log(log(abs(elt), abs(x0)), abs(k))
             if is_almost_int(a) and a > 0:
                 if not almosteq(x0**(k**a), elt):  # parity
                     return False
-                return 1 + round(a)
+                return int(1 + round(a))
+        return False
+
+    # v0.14.3+: cyclic infinite sequences
+    def iscyclic(spec):
+        assert len(spec) >= 1
+        *maybe_initial, maybe_repeating = spec
+        if isinstance(maybe_repeating, list):
+            if not maybe_repeating:
+                raise SyntaxError("Expected non-empty list of repeating elements for cyclic sequence.")
+            return True
         return False
 
     # analyze the specification
     if Ellipsis not in spec:  # convenience fallback
-        return (x for x in spec)
+        if iscyclic(spec):
+            raise SyntaxError("Expected final ... for cyclic sequence.")
+        return imathify(x for x in spec)
     else:
         *spec, last = spec
         if last is Ellipsis:
-            seqtype, x0, k = analyze(*spec)
-            n = infty
+            if not spec:
+                raise SyntaxError("Expected s(a0, a1, ...), s(a0, a1, ..., an), s([*repeats], ...), or s(*initials, [*repeats], ...); got '{}'".format(origspec))
+            assert spec  # not empty
+            # v0.14.3+: cyclic infinite sequences
+            if iscyclic(spec):
+                seqtype = "cyclic"
+                *initial, repeating = spec
+                n = infty
+            else:
+                seqtype, x0, k = analyze(*spec)
+                n = infty
         else:
             *spec, dots = spec
-            if dots is not Ellipsis:
-                raise SyntaxError("Expected s(a0, a1, ...) or s(a0, a1, ..., an); got '{}'".format(origspec))
+            if not (dots is Ellipsis and spec):
+                raise SyntaxError("Expected s(a0, a1, ...) or s(a0, a1, ..., an), s([*repeats], ...), or s(*initials, [*repeats], ...); got '{}'".format(origspec))
+            assert spec  # not empty
             desc = analyze(*spec)
             n = nofterms(desc, last)
             if n is False:
@@ -339,27 +433,31 @@ def s(*spec):
             elif n is infty:
                 raise SyntaxError("The length of a constant sequence cannot be determined from a final element; got '{}'".format(origspec))
             seqtype, x0, k = desc
-        if not spec:
-            raise SyntaxError("Expected at least one term before the '...'; got '{}'".format(origspec))
 
     # generate the sequence
     if seqtype == "const":
-        return m(repeat(x0) if n is infty else repeat(x0, n))
+        return imathify(repeat(x0) if n is infty else repeat(x0, n))
+    elif seqtype == "cyclic":
+        def cyclic():
+            yield from initial
+            while True:
+                yield from repeating
+        return imathify(cyclic())
     elif seqtype == "arith":
         # itertools.count doesn't avoid accumulating roundoff error for floats, so we implement our own.
         # This should be, for any j, within 1 ULP of the true result.
         def arith():
             j = 0
             while True:
-                yield x0 + j*k
+                yield x0 + j * k
                 j += 1
-        return m(arith() if n is infty else take(n, arith()))
+        return imathify(arith() if n is infty else take(n, arith()))
     elif seqtype == "geom":
         if isinstance(k, _symExpr) or abs(k) >= 1:
-            def geom():
+            def geoimathify():
                 j = 0
                 while True:
-                    yield x0*(k**j)
+                    yield x0 * (k**j)
                     j += 1
         else:
             # e.g. "3" can be represented exactly as a base-2 float, but "1/3" can't,
@@ -367,14 +465,14 @@ def s(*spec):
             #
             # Note that 1/(1/3) --> 3.0 even for floats, so we don't actually
             # need to modify the detection algorithm to account for this.
-            kinv = 1/k
-            def geom():
+            kinv = 1 / k
+            def geoimathify():
                 j = 0
                 while True:
-                    yield x0/(kinv**j)
+                    yield x0 / (kinv**j)
                     j += 1
-        return m(geom() if n is infty else take(n, geom()))
-    else: # seqtype == "power":
+        return imathify(geoimathify() if n is infty else take(n, geoimathify()))
+    else:  # seqtype == "power":
         if isinstance(k, _symExpr) or abs(k) >= 1:
             def power():
                 j = 0
@@ -382,18 +480,17 @@ def s(*spec):
                     yield x0**(k**j)
                     j += 1
         else:
-            kinv = 1/k
+            kinv = 1 / k
             def power():
                 j = 0
                 while True:
-                    yield x0**(1/(kinv**j))
+                    yield x0**(1 / (kinv**j))
                     j += 1
-        return m(power() if n is infty else take(n, power()))
+        return imathify(power() if n is infty else take(n, power()))
 
 # -----------------------------------------------------------------------------
 
-# https://docs.python.org/3/reference/datamodel.html#emulating-numeric-types
-class m:
+class imathify:
     """Endow any iterable with infix math support (termwise).
 
     The original iterable is saved to an attribute, and ``m.__iter__`` redirects
@@ -431,13 +528,21 @@ class m:
         e = take(5, c)     # general iterable operation drops math support...
         assert not isinstance(e, m)
 
-        f = m(take(5, c))  # ...and it can be restored by m'ing again.
+        f = imathify(take(5, c))  # ...and it can be restored by m'ing again.
         assert isinstance(f, m)
+
+        g = imathify((1, 2, 3, 4, 5))
+        h = imathify((2, 3, 4, 5, 6))
+        assert tuple(g + h) == (3, 5, 7, 9, 11)
+
+    See the relevant part of the Python language reference:
+
+        https://docs.python.org/3/reference/datamodel.html#emulating-numeric-types
     """
     def __init__(self, iterable):
         self._g = iterable
     def __iter__(self):
-        return self._g
+        return iter(self._g)
     def __add__(self, other):
         return sadd(self, other)
     def __radd__(self, other):
@@ -449,7 +554,7 @@ class m:
     def __abs__(self):
         return sabs(self)
     def __pos__(self):
-        return self
+        return spos(self)
     def __neg__(self):
         return sneg(self)
     def __invert__(self):
@@ -506,92 +611,105 @@ class m:
         return sor(self, other)
     def __ror__(self, other):
         return sor(other, self)
-    # TODO: conversion (bool, complex, int, float) and comparison operators? Do we want those?
+    # Can't do this because each of these conversion operators must return an
+    # instance of that primitive type.
+    # def __bool__(self):
+    #     return sbool(self)
+    # def __complex__(self):
+    #     return scomplex(self)
+    # def __int__(self):
+    #     return sint(self)
+    # def __float__(self):
+    #     return sfloat(self)
+    def __lt__(self, other):
+        return slt(self, other)
+    def __le__(self, other):
+        return sle(self, other)
+    def __eq__(self, other):
+        return seq(self, other)
+    def __ne__(self, other):
+        return sne(self, other)
+    def __ge__(self, other):
+        return sge(self, other)
+    def __gt__(self, other):
+        return sgt(self, other)
 
-def mg(gfunc):
-    """Decorator: make gfunc m() the returned generator instances.
+class m(imathify):  # pragma: no cover
+    """Alias for `imathify`, for backward compatibility.
+
+    Will be removed in 0.15.0."""
+    def __init__(self, iterable):
+        warn("`m` has been renamed `imathify`, which is more descriptive; this alias will be removed in 0.15.0.", FutureWarning)
+        super().__init__(iterable)
+
+def gmathify(gfunc):
+    """Decorator: make gfunc imathify() the returned generator instances.
 
     Return a new gfunc, which passes all its arguments to the original ``gfunc``.
 
     Example::
 
-        a = mg(imemoize(s(1, 2, ...)))
+        a = gmathify(imemoize(s(1, 2, ...)))
         assert last(take(5, a())) == 5
         assert last(take(5, a())) == 5
         assert last(take(5, a() + a())) == 10
     """
     @wraps(gfunc)
     def mathify(*args, **kwargs):
-        return m(gfunc(*args, **kwargs))
+        return imathify(gfunc(*args, **kwargs))
     return mathify
 
+def mg(gfunc):  # pragma: no cover
+    """Alias for `gmathify`, for backward compatibility.
+
+    Will be removed in 0.15.0.
+    """
+    warn("`mg` has been renamed `gmathify`, which is more descriptive; this alias will be removed in 0.15.0.", FutureWarning)
+    return gmathify(gfunc)
+
+# -----------------------------------------------------------------------------
+# We expose the full set of "imathify" operators also as functions à la the ``operator`` module.
+# Prefix "s", short for "mathematical Sequence".
+# https://docs.python.org/3/library/operator.html
+
 # The *settings mechanism is used by round and pow.
 # These are recursive to support iterables containing iterables (e.g. an iterable of math sequences).
 def _make_termwise_stream_unop(op, *settings):
-    def sop(a):
+    def stream_op(a):
         if hasattr(a, "__iter__"):
-            return m(sop(x, *settings) for x in a)
+            return imathify(stream_op(x) for x in a)
         return op(a, *settings)
-    return sop
+    return stream_op
 def _make_termwise_stream_binop(op, *settings):
-    def sop(a, b):
-        ig = [hasattr(x, "__iter__") for x in (a, b)]
-        if all(ig):
+    def stream_op(a, b):
+        isiterable = [hasattr(x, "__iter__") for x in (a, b)]
+        if all(isiterable):
             # it's very convenient here that zip() terminates when the shorter input runs out.
-            return m(sop(x, y, *settings) for x, y in zip(a, b))
-        elif ig[0]:
+            return imathify(stream_op(x, y) for x, y in zip(a, b))
+        elif isiterable[0]:
             c = b
-            return m(sop(x, c, *settings) for x in a)
-        elif ig[1]:
+            return imathify(stream_op(x, c) for x in a)
+        elif isiterable[1]:
             c = a
-            return m(sop(c, y, *settings) for y in b)  # careful; op might not be commutative
-        else: # not any(ig):
+            return imathify(stream_op(c, y) for y in b)  # careful; op might not be commutative
+        else:  # not any(isiterable):
             return op(a, b, *settings)
-    return sop
+    return stream_op
+
+sadd = _make_termwise_stream_binop(primitive_add)
+sadd.__doc__ = """Termwise a + b when one or both are iterables."""
+ssub = _make_termwise_stream_binop(primitive_sub)
+ssub.__doc__ = """Termwise a - b when one or both are iterables."""
+sabs = _make_termwise_stream_unop(abs)
+sabs.__doc__ = """Termwise abs(a) for an iterable."""
+spos = _make_termwise_stream_unop(primitive_pos)
+spos.__doc__ = """Termwise +a for an iterable."""
+sneg = _make_termwise_stream_unop(primitive_neg)
+sneg.__doc__ = """Termwise -a for an iterable."""
+smul = _make_termwise_stream_binop(primitive_mul)
+smul.__doc__ = """Termwise a * b when one or both are iterables."""
 
-# We expose the full set of "m" operators also as functions à la the ``operator`` module.
-_add = _make_termwise_stream_binop(primitive_add)
-_sub = _make_termwise_stream_binop(primitive_sub)
-_abs = _make_termwise_stream_unop(abs)
-_pos = _make_termwise_stream_unop(primitive_pos)
-_neg = _make_termwise_stream_unop(primitive_neg)
-_mul = _make_termwise_stream_binop(primitive_mul)
 _pow = _make_termwise_stream_binop(primitive_pow)  # 2-arg form
-_truediv = _make_termwise_stream_binop(primitive_truediv)
-_floordiv = _make_termwise_stream_binop(primitive_floordiv)
-_mod = _make_termwise_stream_binop(primitive_mod)
-_divmod = _make_termwise_stream_binop(divmod)
-_round = _make_termwise_stream_unop(round)  # 1-arg form
-_trunc = _make_termwise_stream_unop(trunc)
-_floor = _make_termwise_stream_unop(floor)
-_ceil = _make_termwise_stream_unop(ceil)
-_lshift = _make_termwise_stream_binop(primitive_lshift)
-_rshift = _make_termwise_stream_binop(primitive_rshift)
-_and = _make_termwise_stream_binop(primitive_and)
-_xor = _make_termwise_stream_binop(primitive_xor)
-_or = _make_termwise_stream_binop(primitive_or)
-_invert = _make_termwise_stream_unop(primitive_invert)
-def sadd(a, b):
-    """Termwise a + b when one or both are iterables."""
-    return _add(a, b)
-def ssub(a, b):
-    """Termwise a - b when one or both are iterables."""
-    return _sub(a, b)
-def sabs(a):
-    """Termwise abs(a) for an iterable."""
-    return _abs(a)
-def spos(a):
-    """Termwise +a for an iterable."""
-    return _pos(a)
-def sneg(a):
-    """Termwise -a for an iterable."""
-    return _neg(a)
-def sinvert(a):
-    """Termwise ~a for an iterable."""
-    return _invert(a)
-def smul(a, b):
-    """Termwise a * b when one or both are iterables."""
-    return _mul(a, b)
 def spow(a, b, *mod):
     """Termwise a ** b when one or both are iterables.
 
@@ -600,50 +718,85 @@ def spow(a, b, *mod):
     """
     op = _make_termwise_stream_binop(pow, mod[0]) if mod else _pow
     return op(a, b)
-def struediv(a, b):
-    """Termwise a / b when one or both are iterables."""
-    return _truediv(a, b)
-def sfloordiv(a, b):
-    """Termwise a // b when one or both are iterables."""
-    return _floordiv(a, b)
-def smod(a, b):
-    """Termwise a % b when one or both are iterables."""
-    return _mod(a, b)
-def sdivmod(a, b):
-    """Termwise (a // b, a % b) when one or both are iterables."""
-    return _divmod(a, b)
+
+struediv = _make_termwise_stream_binop(primitive_truediv)
+struediv.__doc__ = """Termwise a / b when one or both are iterables."""
+sfloordiv = _make_termwise_stream_binop(primitive_floordiv)
+sfloordiv.__doc__ = """Termwise a // b when one or both are iterables."""
+smod = _make_termwise_stream_binop(primitive_mod)
+smod.__doc__ = """Termwise a % b when one or both are iterables."""
+sdivmod = _make_termwise_stream_binop(divmod)
+sdivmod.__doc__ = """Termwise (a // b, a % b) when one or both are iterables."""
+
+_round = _make_termwise_stream_unop(round)  # 1-arg form
 def sround(a, *ndigits):
     """Termwise round(a) for an iterable.
 
     An optional second argument is supported, and passed through to the
     built-in ``round`` function.
+
+    As with the built-in, rounding is correct taking into account the float
+    representation, which is base-2.
+
+        https://docs.python.org/3/library/functions.html#round
     """
     op = _make_termwise_stream_unop(round, ndigits[0]) if ndigits else _round
     return op(a)
-def strunc(a):
-    """Termwise math.trunc(a) for an iterable."""
-    return _trunc(a)
-def sfloor(a):
-    """Termwise math.floor(a) for an iterable."""
-    return _floor(a)
-def sceil(a):
-    """Termwise math.ceil(a) for an iterable."""
-    return _ceil(a)
-def slshift(a, b):
-    """Termwise a << b when one or both are iterables."""
-    return _lshift(a, b)
-def srshift(a, b):
-    """Termwise a >> b when one or both are iterables."""
-    return _rshift(a, b)
-def sand(a, b):
-    """Termwise a & b when one or both are iterables."""
-    return _and(a, b)
-def sxor(a, b):
-    """Termwise a ^ b when one or both are iterables."""
-    return _xor(a, b)
-def sor(a, b):
-    """Termwise a | b when one or both are iterables."""
-    return _or(a, b)
+
+strunc = _make_termwise_stream_unop(trunc)
+strunc.__doc__ = """Termwise math.trunc(a) for an iterable."""
+sfloor = _make_termwise_stream_unop(floor)
+sfloor.__doc__ = """Termwise math.floor(a) for an iterable."""
+sceil = _make_termwise_stream_unop(ceil)
+sceil.__doc__ = """Termwise math.ceil(a) for an iterable."""
+
+# bit twiddling operations
+slshift = _make_termwise_stream_binop(primitive_lshift)
+slshift.__doc__ = """Termwise a << b when one or both are iterables."""
+srshift = _make_termwise_stream_binop(primitive_rshift)
+srshift.__doc__ = """Termwise a >> b when one or both are iterables."""
+sand = _make_termwise_stream_binop(primitive_and)
+sand.__doc__ = """Termwise a & b when one or both are iterables."""
+sxor = _make_termwise_stream_binop(primitive_xor)
+sxor.__doc__ = """Termwise a ^ b when one or both are iterables."""
+sor = _make_termwise_stream_binop(primitive_or)
+sor.__doc__ = """Termwise a | b when one or both are iterables."""
+sinvert = _make_termwise_stream_unop(primitive_invert)
+sinvert.__doc__ = """Termwise ~a for an iterable.
+
+Note this is a bitwise invert, which is usually not what you want.
+
+However, there is no ``__not__`` method for object instances,
+only the interpreter core defines that operation.
+
+See:
+    https://docs.python.org/3/library/operator.html#operator.not_
+"""
+
+# Can't do this because each of these conversion operators must return an
+# instance of that primitive type.
+#
+# sbool = _make_termwise_stream_unop(bool)
+# sbool.__doc__ = """Termwise bool(a) for an iterable."""
+# scomplex = _make_termwise_stream_unop(complex)
+# scomplex.__doc__ = """Termwise complex(a) for an iterable."""
+# sint = _make_termwise_stream_unop(int)
+# sint.__doc__ = """Termwise int(a) for an iterable."""
+# sfloat = _make_termwise_stream_unop(float)
+# sfloat.__doc__ = """Termwise float(a) for an iterable."""
+
+slt = _make_termwise_stream_binop(primitive_lt)
+slt.__doc__ = """Termwise a < b when one or both are iterables."""
+sle = _make_termwise_stream_binop(primitive_le)
+sle.__doc__ = """Termwise a <= b when one or both are iterables."""
+seq = _make_termwise_stream_binop(primitive_eq)
+seq.__doc__ = """Termwise a == b when one or both are iterables."""
+sne = _make_termwise_stream_binop(primitive_ne)
+sne.__doc__ = """Termwise a != b when one or both are iterables."""
+sge = _make_termwise_stream_binop(primitive_ge)
+sge.__doc__ = """Termwise a >= b when one or both are iterables."""
+sgt = _make_termwise_stream_binop(primitive_gt)
+sgt.__doc__ = """Termwise a > b when one or both are iterables."""
 
 # -----------------------------------------------------------------------------
 
@@ -652,7 +805,7 @@ def cauchyprod(a, b, *, require="any"):
 
     Defined by::
 
-        c[k] = sum(a[j] * b[k-j], j = 0, 1, ..., k),  k = 0, 1, ...
+        c[k] = suimathify(a[j] * b[k-j], j = 0, 1, ..., k),  k = 0, 1, ...
 
     As a table::
 
@@ -759,7 +912,7 @@ def diagonal_reduce(a, b, *, combine, reduce, require="any"):
     contribute to it in the infinite case) cannot be formed from length-2 inputs.
     """
     if not all(hasattr(x, "__iter__") for x in (a, b)):
-        raise TypeError("Expected two iterables, got '{}', '{}'".format(type(a), type(b)))
+        raise TypeError("Expected two iterables, got {}, {}".format(type(a), type(b)))
     if require not in ("all", "any"):
         raise ValueError("require must be 'all' or 'any'; got '{}'".format(require))
     ga = imemoize(a)
@@ -779,7 +932,7 @@ def diagonal_reduce(a, b, *, combine, reduce, require="any"):
                 break
             yield reduce(combine(xs, rev(ys)))
             n += 1
-    return m(diagonal())
+    return imathify(diagonal())
 
 # -----------------------------------------------------------------------------
 
@@ -790,7 +943,7 @@ def fibonacci():
         while True:
             yield a
             a, b = b, a + b
-    return m(fibos())
+    return imathify(fibos())
 
 # See test_gmemo.py for history. This is an FP-ized sieve of Eratosthenes.
 #
@@ -803,7 +956,7 @@ def fibonacci():
 def _primes():
     yield 2
     for n in count(start=3, step=2):
-        if not any(n % p == 0 for p in takewhile(lambda x: x*x <= n, _primes())):
+        if not any(n % p == 0 for p in takewhile(lambda x: x * x <= n, _primes())):
             yield n
 
 @gmemoize
@@ -813,7 +966,7 @@ def _fastprimes():
         memo.append(2)
         yield 2
         for n in count(start=3, step=2):
-            if not any(n % p == 0 for p in takewhile(lambda x: x*x <= n, memo)):
+            if not any(n % p == 0 for p in takewhile(lambda x: x * x <= n, memo)):
                 memo.append(n)
                 yield n
     return primes()
@@ -832,6 +985,6 @@ def primes(optimize="speed"):
     if optimize not in ("memory", "speed"):
         raise ValueError("optimize must be 'memory' or 'speed'; got '{}'".format(optimize))
     if optimize == "speed":
-        return m(_fastprimes())
-    else: # optimize == "memory":
-        return m(_primes())
+        return imathify(_fastprimes())
+    else:  # optimize == "memory":
+        return imathify(_primes())
