@@ -3,16 +3,16 @@
 
 __all__ = ["begin", "begin0", "lazy_begin", "lazy_begin0",
            "pipe1", "piped1", "lazy_piped1",
-           "pipe", "piped", "getvalue", "lazy_piped", "runpipe",
+           "pipe", "piped", "getvalue", "lazy_piped", "runpipe", "exitpipe",
            "pipec",  # w/ curry
            "do", "do0", "assign"]
 
 from collections import namedtuple
 from .env import env
-from .misc import call
 from .fun import curry, iscurried
 from .dynassign import dyn
 from .arity import arity_includes, UnknownArity
+from .singleton import Singleton
 
 # sequence side effects in a lambda
 def begin(*vals):
@@ -47,10 +47,10 @@ def lazy_begin(*bodys):
                                  lambda: 42*x)
         print(f(1))  # 42
     """
-    l = len(bodys)
-    if not l:
+    n = len(bodys)
+    if not n:
         return None
-    if l == 1:
+    if n == 1:
         b = bodys[0]
         return b()
     *rest, last = bodys
@@ -68,10 +68,10 @@ def lazy_begin0(*bodys):
                                   lambda: print("hi"))
         print(g(1))  # 23
     """
-    l = len(bodys)
-    if not l:
+    n = len(bodys)
+    if not n:
         return None
-    if l == 1:
+    if n == 1:
         b = bodys[0]
         return b()
     first, *rest = bodys
@@ -125,24 +125,34 @@ def pipe1(value0, *bodys):
 
     at the cost of namespace pollution.
     """
-    # Ideally we should use fploop, but we choose to cheat imperatively to avoid
-    # the added complexity of supporting the runtime-switchable TCO implementation.
-#    @looped_over(bodys, acc=value0)
-#    def x(loop, update, acc):
-#        return loop(update(acc))
-#    return x
-    # Since "x" is a local, the imperative damage won't spread to the call site.
+    # Ideally we should use fploop, but we choose to cheat imperatively.
+    # This used to be to avoid the added complexity of supporting the
+    # runtime-switchable TCO implementation, but ever since we got rid of that
+    # misfeature all the way back in v0.10, there's been really no reason to
+    # avoid @looped_over, except performance.
+    #
+    # Since "x" is a local, the imperative damage won't spread to the call
+    # site. So we can just as well use the builtin imperative for loop, and
+    # reap the performance benefit.
+    # @looped_over(bodys, acc=value0)
+    # def x(loop, update, acc):
+    #     return loop(update(acc))
+    # return x
     x = value0
     for update in bodys:
         x = update(x)
     return x
 
-@call  # make a singleton
-class getvalue:  # sentinel with a nice repr
+class Getvalue(Singleton):  # singleton sentinel with a nice repr
     """Sentinel; pipe into this to exit a shell-like pipe and return the current value."""
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<sentinel for pipe exit>"
-runpipe = getvalue  # same thing, but semantically better name for lazy pipes
+getvalue = Getvalue()
+runpipe = getvalue  # same thing as getvalue, but semantically better name for lazy pipes
+
+# New unified name for v0.15.0; deprecating the separate "getvalue" and "runpipe" as of v0.14.2.
+# TODO: Now that we have symbols, in v0.15.0, change this to `sym("exitpipe")` and delete the `Getvalue` class.
+exitpipe = getvalue
 
 class piped1:
     """Shell-like piping syntax.
@@ -157,7 +167,7 @@ class piped1:
 
         Return a ``piped`` object, for chainability.
 
-        As the only exception, if ``f`` is the sentinel ``getvalue``,
+        As the only exception, if ``f`` is the sentinel ``exitpipe``,
         return the current value (useful for exiting the pipe).
 
         A new ``piped`` object is created at each step of piping; the "update"
@@ -165,17 +175,17 @@ class piped1:
 
         Examples::
 
-            x = piped1(42) | double | inc | getvalue
+            x = piped1(42) | double | inc | exitpipe
 
             y = piped1(42) | double
-            assert y | inc | getvalue == 85
-            assert y | getvalue == 84  # y is not modified
+            assert y | inc | exitpipe == 85
+            assert y | exitpipe == 84  # y is not modified
         """
-        if f is getvalue:
+        if f is exitpipe:
             return self._x
         cls = self.__class__
         return cls(f(self._x))  # functional update
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<piped1 at 0x{:x}; value {}>".format(id(self), self._x)
 
 class lazy_piped1:
@@ -188,7 +198,7 @@ class lazy_piped1:
           the pipeline.
 
         - ``lazy_piped`` just sets up a computation, and performs it when eventually
-          piped into ``runpipe``. The computation always looks up the latest state
+          piped into ``exitpipe``. The computation always looks up the latest state
           of the initial value.
 
     Another way to say this is that ``lazy_piped`` looks up the initial value
@@ -204,7 +214,7 @@ class lazy_piped1:
     def __or__(self, f):
         """Pipe the value into f; but just plan to do so, don't perform it yet.
 
-        To run the stored computation, pipe into ``runpipe``.
+        To run the stored computation, pipe into ``exitpipe``.
 
         Examples::
 
@@ -214,7 +224,7 @@ class lazy_piped1:
                 return l  # important, handed to the next function in the pipe
             p = lazy_piped1(lst) | append_succ | append_succ  # plan a computation
             assert lst == [1]        # nothing done yet
-            p | runpipe              # run the computation
+            p | exitpipe              # run the computation
             assert lst == [1, 2, 3]  # now the side effect has updated lst.
 
             # lazy pipe as an unfold
@@ -226,10 +236,10 @@ class lazy_piped1:
             p = lazy_piped1((1, 1))  # load initial state into a lazy pipe
             for _ in range(10):      # set up pipeline
                 p = p | nextfibo
-            p | runpipe
+            p | exitpipe
             print(fibos)
         """
-        if f is runpipe:  # compute now
+        if f is exitpipe:  # compute now
             v = self._x
             for g in self._funcs:
                 v = g(v)
@@ -237,7 +247,7 @@ class lazy_piped1:
         # just pass on the reference to the original x.
         cls = self.__class__
         return cls(x=self._x, _funcs=self._funcs + (f,))
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<lazy_piped1 at 0x{:x}; initial value now {}, functions {}>".format(id(self), self._x, self._funcs)
 
 def pipe(values0, *bodys):
@@ -314,20 +324,18 @@ class piped:
 
             f = lambda x, y: (2*x, y+1)
             g = lambda x, y: (x+1, 2*y)
-            x = piped(2, 3) | f | g | getvalue  # --> (5, 8)
+            x = piped(2, 3) | f | g | exitpipe  # --> (5, 8)
         """
         xs = self._xs
-        if f is getvalue:
+        if f is exitpipe:
             return xs if len(xs) > 1 else xs[0]
         cls = self.__class__
-        if isinstance(xs, tuple):
-            newxs = f(*xs)
-        else:
-            newxs = f(xs)
+        assert isinstance(xs, tuple)  # __init__ ensures this
+        newxs = f(*xs)
         if isinstance(newxs, tuple):
             return cls(*newxs)
         return cls(newxs)
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<piped at 0x{:x}; values {}>".format(id(self), self._xs)
 
 class lazy_piped:
@@ -340,7 +348,7 @@ class lazy_piped:
         p3 = p2 | (lambda x, y, s: (x * 2, y + 1, "got {}".format(s)))
         p4 = p3 | (lambda x, y, s: (x + y, s))
         # nothing done yet!
-        assert (p4 | runpipe) == (13, "got foo")
+        assert (p4 | exitpipe) == (13, "got foo")
 
         # lazy pipe as an unfold
         fibos = []
@@ -350,7 +358,7 @@ class lazy_piped:
         p = lazy_piped(1, 1)
         for _ in range(10):
             p = p | nextfibo
-        p | runpipe
+        p | exitpipe
         print(fibos)
     """
     def __init__(self, *xs, _funcs=None):
@@ -362,18 +370,21 @@ class lazy_piped:
         self._funcs = _funcs or ()
     def __or__(self, f):
         """Pipe the values into f; but just plan to do so, don't perform it yet."""
-        if f is runpipe:  # compute now
+        if f is exitpipe:  # compute now
             vs = self._xs
             for g in self._funcs:
                 if isinstance(vs, tuple):
                     vs = g(*vs)
                 else:
                     vs = g(vs)
-            return vs if len(vs) > 1 else vs[0]
+            if isinstance(vs, tuple):
+                return vs if len(vs) > 1 else vs[0]
+            else:
+                return vs
         # just pass on the references to the original xs.
         cls = self.__class__
         return cls(*self._xs, _funcs=self._funcs + (f,))
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<lazy_piped at 0x{:x}; initial values now {}, functions {}>".format(id(self), self._xs, self._funcs)
 
 # do(): improved begin() that can name intermediate results and refer to them
@@ -475,7 +486,7 @@ def do(*items):
             try:
                 if not arity_includes(v, 1):
                     raise ValueError("Arity mismatch; callable value must allow arity 1, to take in the environment.")
-            except UnknownArity:  # well, we tried!
+            except UnknownArity:  # well, we tried!  # pragma: no cover
                 pass
             return v(e)
         return v
