@@ -5,25 +5,25 @@ The common factor is tail-position analysis."""
 
 from functools import partial
 
-from ast import Lambda, FunctionDef, \
-                arguments, arg, keyword, \
-                List, Tuple, \
-                Subscript, Index, \
-                Call, Name, Starred, NameConstant, \
-                BoolOp, And, Or, \
-                With, If, IfExp, Try, Assign, Return, Expr, \
-                copy_location
+from ast import (Lambda, FunctionDef,
+                 arguments, arg, keyword,
+                 List, Tuple,
+                 Subscript, Index,
+                 Call, Name, Starred, NameConstant,
+                 BoolOp, And, Or,
+                 With, If, IfExp, Try, Assign, Return, Expr,
+                 copy_location)
 from .astcompat import AsyncFunctionDef, AsyncWith
 
 from macropy.core.macros import macro_stub
 from macropy.core.quotes import macros, q, u, ast_literal, name
-from macropy.core.hquotes import macros, hq
+from macropy.core.hquotes import macros, hq  # noqa: F811, F401
 from macropy.core.walkers import Walker
 
-from .util import isx, make_isxpred, isec, \
-                  detect_callec, detect_lambda, \
-                  has_tco, sort_lambda_decorators, \
-                  suggest_decorator_index, ContinuationsMarker, wrapwith, ismarker
+from .util import (isx, make_isxpred, isec,
+                   detect_callec, detect_lambda,
+                   has_tco, sort_lambda_decorators,
+                   suggest_decorator_index, ContinuationsMarker, wrapwith, ismarker)
 from .letdoutil import isdo, islet, ExpandedLetView, ExpandedDoView
 from .ifexprs import aif
 
@@ -31,7 +31,7 @@ from ..dynassign import dyn
 from ..it import uniqify
 from ..fun import identity, orf
 from ..tco import trampolined, jump
-from ..lazyutil import mark_lazy
+from ..lazyutil import passthrough_lazy_args
 
 # -----------------------------------------------------------------------------
 # Implicit return. This performs a tail-position analysis of function bodies.
@@ -122,7 +122,7 @@ def call_cc(tree, **kw):
 
     For more, see the docstring of ``continuations``.
     """
-    pass
+    pass  # pragma: no cover
 
 # _pcc/cc chaining handler, to be exported to client code via hq[].
 #
@@ -134,7 +134,7 @@ def chain_conts(cc1, cc2, with_star=False):  # cc1=_pcc, cc2=cc
     """Internal function, used in code generated by the continuations macro."""
     if with_star:  # to be chainable from a tail call, accept a multiple-values arglist
         if cc1 is not None:
-            @mark_lazy
+            @passthrough_lazy_args
             def cc(*value):
                 return jump(cc1, cc=cc2, *value)
         else:
@@ -145,14 +145,14 @@ def chain_conts(cc1, cc2, with_star=False):  # cc1=_pcc, cc2=cc
             cc = cc2
     else:  # for inert data value returns (this produces the multiple-values arglist)
         if cc1 is not None:
-            @mark_lazy
+            @passthrough_lazy_args
             def cc(value):
                 if isinstance(value, tuple):
                     return jump(cc1, cc=cc2, *value)
                 else:
                     return jump(cc1, value, cc=cc2)
         else:
-            @mark_lazy
+            @passthrough_lazy_args
             def cc(value):
                 if isinstance(value, tuple):
                     return jump(cc2, *value)
@@ -196,7 +196,7 @@ def continuations(block_body):
             nd = len(tree.args.defaults)  # defaults apply to n last args
             if j == na - nd - 1:  # last one that has no default
                 tree.args.defaults.insert(0, hq[identity])
-        else: # "cc" in kwonlynames:
+        else:  # "cc" in kwonlynames:
             j = kwonlynames.index("cc")
             if tree.args.kw_defaults[j] is None:  # not already set
                 tree.args.kw_defaults[j] = hq[identity]
@@ -255,11 +255,10 @@ def continuations(block_body):
         if type(tree) not in (Assign, Expr):
             return False
         tree = tree.value
-        if type(tree) is Subscript and type(tree.value) is Name \
-           and tree.value.id == "call_cc":
+        if type(tree) is Subscript and type(tree.value) is Name and tree.value.id == "call_cc":
             if type(tree.slice) is Index:
                 return True
-            assert False, "expected single expr, not slice in call_cc[...]"
+            assert False, "expected single expr, not slice in call_cc[...]"  # pragma: no cover
         return False
     def split_at_callcc(body):
         if not body:
@@ -271,7 +270,13 @@ def continuations(block_body):
                 # after is always non-empty here (has at least the explicitified "return")
                 # ...unless we're at the top level of the "with continuations" block
                 if not after:
-                    assert False, "call_cc[] cannot appear as the last statement of a 'with continuations' block (no continuation to capture)"
+                    assert False, "call_cc[] cannot appear as the last statement of a 'with continuations' block (no continuation to capture)"  # pragma: no cover
+                # TODO: To support Python's scoping properly in assignments after the `call_cc`,
+                # TODO: we have to scan `before` for assignments to local variables (stopping at
+                # TODO: scope boundaries; use `get_names_in_store_context` from our `scoping` module),
+                # TODO: and declare those variables `nonlocal` in `after`. This way the binding
+                # TODO: will be shared between the original context and the continuation.
+                # See Politz et al 2013 (the "full monty" paper), section 4.2.
                 return before, stmt, after
             before.append(stmt)
             if not after:
@@ -284,42 +289,42 @@ def continuations(block_body):
                 return [expr.id]
             elif type(expr) is Starred:
                 if type(expr.value) is not Name:
-                    assert False, "call_cc[] starred assignment target must be a bare name"
+                    assert False, "call_cc[] starred assignment target must be a bare name"  # pragma: no cover
                 starget = expr.value.id
                 return []
-            assert False, "all call_cc[] assignment targets must be bare names (last one may be starred)"
+            assert False, "all call_cc[] assignment targets must be bare names (last one may be starred)"  # pragma: no cover
         # extract the assignment targets (args of the cont)
         if type(stmt) is Assign:
             if len(stmt.targets) != 1:
-                assert False, "expected at most one '=' in a call_cc[] statement"
+                assert False, "expected at most one '=' in a call_cc[] statement"  # pragma: no cover
             target = stmt.targets[0]
             if type(target) in (Tuple, List):
                 rest, last = target.elts[:-1], target.elts[-1]
                 # TODO: limitation due to Python's vararg syntax - the "*args" must be after positional args.
                 if any(type(x) is Starred for x in rest):
-                    assert False, "in call_cc[], only the last assignment target may be starred"
+                    assert False, "in call_cc[], only the last assignment target may be starred"  # pragma: no cover
                 if not all(type(x) is Name for x in rest):
-                    assert False, "all call_cc[] assignment targets must be bare names"
+                    assert False, "all call_cc[] assignment targets must be bare names"  # pragma: no cover
                 targets = [x.id for x in rest] + maybe_starred(last)
             else:  # single target
                 targets = maybe_starred(target)
         elif type(stmt) is Expr:  # no assignment targets, cont takes no args
             targets = []
         else:
-            assert False, "call_cc[]: expected an assignment or a bare expr, got {}".format(stmt)
+            assert False, "call_cc[]: expected an assignment or a bare expr, got {}".format(stmt)  # pragma: no cover
         # extract the function call(s)
         if type(stmt.value) is not Subscript:  # both Assign and Expr have a .value
-            assert False, "expected either an assignment with a call_cc[] expr on RHS, or a bare call_cc[] expr, got {}".format(stmt.value)
+            assert False, "expected either an assignment with a call_cc[] expr on RHS, or a bare call_cc[] expr, got {}".format(stmt.value)  # pragma: no cover
         theexpr = stmt.value.slice.value
         if not (type(theexpr) in (Call, IfExp) or (type(theexpr) is NameConstant and theexpr.value is None)):
-            assert False, "the bracketed expression in call_cc[...] must be a function call, an if-expression, or None"
+            assert False, "the bracketed expression in call_cc[...] must be a function call, an if-expression, or None"  # pragma: no cover
         def extract_call(tree):
             if type(tree) is Call:
                 return tree
             elif type(tree) is NameConstant and tree.value is None:
                 return None
             else:
-                assert False, "call_cc[...]: expected a function call or None"
+                assert False, "call_cc[...]: expected a function call or None"  # pragma: no cover
         if type(theexpr) is IfExp:
             condition = theexpr.test
             thecall = extract_call(theexpr.body)
@@ -363,7 +368,7 @@ def continuations(block_body):
             if tree:
                 tree.keywords = [keyword(arg="cc", value=q[name[contname]])] + tree.keywords
             else:  # no call means proceed to cont directly, with args set to None
-                tree = q[name[contname](*([None]*u[len(targets)]), cc=name["cc"])]
+                tree = q[name[contname](*([None] * u[len(targets)]), cc=name["cc"])]
             return tree
         thecall = prepare_call(thecall)
         if condition:
@@ -435,11 +440,11 @@ def continuations(block_body):
     @Walker
     def check_for_strays(tree, **kw):
         if iscallcc(tree):
-            assert False, "call_cc[...] only allowed at the top level of a def or async def, or at the top level of the block; must appear as an expr or an assignment RHS"
+            assert False, "call_cc[...] only allowed at the top level of a def or async def, or at the top level of the block; must appear as an expr or an assignment RHS"  # pragma: no cover
         if type(tree) in (Assign, Expr):
             v = tree.value
             if type(v) is Call and type(v.func) is Name and v.func.id == "call_cc":
-                assert False, "call_cc(...) should be call_cc[...] (note brackets; it's a macro)"
+                assert False, "call_cc(...) should be call_cc[...] (note brackets; it's a macro)"  # pragma: no cover
         return tree
 
     # -------------------------------------------------------------------------
@@ -451,7 +456,7 @@ def continuations(block_body):
     # invocation. (Because call_cc[] internally creates a function and calls it.)
     for stmt in block_body:
         if type(stmt) is Return:
-            assert False, "'return' not allowed at the top level of a 'with continuations' block"
+            assert False, "'return' not allowed at the top level of a 'with continuations' block"  # pragma: no cover
 
     # Since we transform **all** returns (even those with an inert data value)
     # into tail calls (to cc), we must insert any missing implicit "return"
@@ -519,12 +524,11 @@ def _tco_transform_def(tree, *, preproc_cb, **kw):
 # transform_retexpr: return-value expression transformer (for TCO and stuff).
 @Walker
 def _tco_transform_return(tree, *, known_ecs, transform_retexpr, **kw):
-    treeisec = isec(tree, known_ecs)
     if type(tree) is Return:
         non = q[None]
         non = copy_location(non, tree)
         value = tree.value or non  # return --> return None  (bare return has value=None in the AST)
-        if not treeisec:
+        if not isec(value, known_ecs):
             return Return(value=transform_retexpr(value, known_ecs))
         else:
             # An ec call already escapes, so the return is redundant.
@@ -533,9 +537,9 @@ def _tco_transform_return(tree, *, known_ecs, transform_retexpr, **kw):
             # this cleans up the code, since eliminating the "return" allows us
             # to omit a redundant "let".
             return Expr(value=value)  # return ec(...) --> ec(...)
-    elif treeisec:  # TCO the arg of an ec(...) call
+    elif isec(tree, known_ecs):  # TCO the arg of an ec(...) call
         if len(tree.args) > 1:
-            assert False, "expected exactly one argument for escape continuation"
+            assert False, "expected exactly one argument for escape continuation"  # pragma: no cover
         tree.args[0] = transform_retexpr(tree.args[0], known_ecs)
     return tree
 
@@ -647,7 +651,7 @@ def _transform_retexpr(tree, known_ecs, call_cb=None, data_cb=None):
                                                                lineno=tree.lineno,
                                                                col_offset=tree.col_offset)),
                                            transform(tree.values[-1])],
-                                     lineno=tree.lineno, col_offset=tree.col_offset)) # tail-call item
+                                     lineno=tree.lineno, col_offset=tree.col_offset))  # tail-call item
                 elif type(tree.op) is And:
                     # and(data1, ..., datan, tail) --> tail if all(others) else False
                     fal = q[False]
@@ -656,7 +660,7 @@ def _transform_retexpr(tree, known_ecs, call_cb=None, data_cb=None):
                                  body=transform(tree.values[-1]),
                                  orelse=transform_data(fal))
                 else:  # cannot happen
-                    assert False, "unknown BoolOp type {}".format(tree.op)
+                    assert False, "unknown BoolOp type {}".format(tree.op)  # pragma: no cover
             else:  # optimization: BoolOp, no call or compound in tail position --> treat as single data item
                 tree = transform_data(tree)
         else:
